@@ -46,7 +46,6 @@
 #include "tiledb/sm/enums/encryption_type.h"
 #include "tiledb/sm/global_state/unit_test_config.h"
 #include "tiledb/sm/misc/tdb_time.h"
-#include "tiledb/sm/misc/utils.h"
 
 #include <iostream>
 #include <sstream>
@@ -85,7 +84,7 @@ struct GroupFx {
   std::string get_golden_walk(const std::string& path);
   std::string get_golden_ls(const std::string& path);
   std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> read_group(
-      tiledb_group_t* group, bool use_get_member_by_index_v2 = true) const;
+      tiledb_group_t* group) const;
   void set_group_timestamp(
       tiledb_group_t* group, const uint64_t& timestamp) const;
   void write_group_metadata(const char* group_uri) const;
@@ -229,49 +228,30 @@ void GroupFx::vacuum(const char* group_uri) const {
 }
 
 std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> GroupFx::read_group(
-    tiledb_group_t* group, bool use_get_member_by_index_v2) const {
+    tiledb_group_t* group) const {
   std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> ret;
   uint64_t count = 0;
   int rc = tiledb_group_get_member_count(ctx_, group, &count);
   REQUIRE(rc == TILEDB_OK);
-  if (use_get_member_by_index_v2) {
-    for (uint64_t i = 0; i < count; i++) {
-      std::string uri;
-      tiledb_object_t type;
-      tiledb_string_t *uri_handle, *name_handle;
-      rc = tiledb_group_get_member_by_index_v2(
-          ctx_, group, i, &uri_handle, &type, &name_handle);
-      REQUIRE(rc == TILEDB_OK);
-      const char* uri_str;
-      size_t uri_size;
-      rc = tiledb_string_view(uri_handle, &uri_str, &uri_size);
+  for (uint64_t i = 0; i < count; i++) {
+    std::string uri;
+    tiledb_object_t type;
+    tiledb_string_t *uri_handle, *name_handle;
+    rc = tiledb_group_get_member_by_index_v2(
+        ctx_, group, i, &uri_handle, &type, &name_handle);
+    REQUIRE(rc == TILEDB_OK);
+    const char* uri_str;
+    size_t uri_size;
+    rc = tiledb_string_view(uri_handle, &uri_str, &uri_size);
+    CHECK(rc == TILEDB_OK);
+    uri = std::string(uri_str, uri_size);
+    rc = tiledb_string_free(&uri_handle);
+    CHECK(rc == TILEDB_OK);
+    if (name_handle) {
+      rc = tiledb_string_free(&name_handle);
       CHECK(rc == TILEDB_OK);
-      uri = std::string(uri_str, uri_size);
-      rc = tiledb_string_free(&uri_handle);
-      CHECK(rc == TILEDB_OK);
-      if (name_handle) {
-        rc = tiledb_string_free(&name_handle);
-        CHECK(rc == TILEDB_OK);
-      }
-      ret.emplace_back(uri, type);
-    }
-  } else {
-    // When tiledb_group_get_member_by_index gets removed, the else part can
-    // simply be removed.
-    for (uint64_t i = 0; i < count; i++) {
-      std::string uri;
-      tiledb_object_t type;
-      char *uri_ptr, *name_ptr;
-      rc = tiledb_group_get_member_by_index(
-          ctx_, group, i, &uri_ptr, &type, &name_ptr);
-      REQUIRE(rc == TILEDB_OK);
-      uri = uri_ptr == nullptr ? "" : std::string(uri_ptr);
-      std::free(uri_ptr);
-      if (name_ptr) {
-        std::free(name_ptr);
-      }
-      ret.emplace_back(uri, type);
     }
+    ret.emplace_back(uri, type);
   }
   return ret;
 }
@@ -570,13 +550,6 @@ TEST_CASE_METHOD(
 
 TEST_CASE_METHOD(
     GroupFx, "C API: Group, write/read", "[capi][group][metadata][read]") {
-  bool use_get_member_by_index_v2 = true;
-  SECTION("Using tiledb_group_get_member_by_index_v2") {
-    use_get_member_by_index_v2 = true;
-  }
-  SECTION("Using tiledb_group_get_member_by_index") {
-    use_get_member_by_index_v2 = false;
-  }
   // Create and open group in write mode
   // TODO: refactor for each supported FS.
   std::string temp_dir = fs_vec_[0]->temp_dir();
@@ -619,18 +592,34 @@ TEST_CASE_METHOD(
   rc = tiledb_group_open(ctx_, group2, TILEDB_WRITE);
   REQUIRE(rc == TILEDB_OK);
 
-  rc =
-      tiledb_group_add_member(ctx_, group1, array1_uri.c_str(), false, nullptr);
-  REQUIRE(rc == TILEDB_OK);
-  rc =
-      tiledb_group_add_member(ctx_, group1, array2_uri.c_str(), false, nullptr);
-  REQUIRE(rc == TILEDB_OK);
-  rc =
-      tiledb_group_add_member(ctx_, group2, array3_uri.c_str(), false, nullptr);
-  REQUIRE(rc == TILEDB_OK);
-  rc =
-      tiledb_group_add_member(ctx_, group1, group2_uri.c_str(), false, nullptr);
-  REQUIRE(rc == TILEDB_OK);
+  bool add_with_type = GENERATE(true, false);
+  if (add_with_type) {
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, array1_uri.c_str(), false, nullptr, TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, array2_uri.c_str(), false, nullptr, TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group2, array3_uri.c_str(), false, nullptr, TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, group2_uri.c_str(), false, nullptr, TILEDB_GROUP);
+    REQUIRE(rc == TILEDB_OK);
+  } else {
+    rc = tiledb_group_add_member(
+        ctx_, group1, array1_uri.c_str(), false, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group1, array2_uri.c_str(), false, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group2, array3_uri.c_str(), false, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group1, group2_uri.c_str(), false, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+  }
 
   // Close group from write mode
   rc = tiledb_group_close(ctx_, group1);
@@ -646,12 +635,12 @@ TEST_CASE_METHOD(
   REQUIRE(rc == TILEDB_OK);
 
   std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> group1_received =
-      read_group(group1, use_get_member_by_index_v2);
+      read_group(group1);
   REQUIRE_THAT(
       group1_received, Catch::Matchers::UnorderedEquals(group1_expected));
 
   std::vector<std::pair<tiledb::sm::URI, tiledb_object_t>> group2_received =
-      read_group(group2, use_get_member_by_index_v2);
+      read_group(group2);
   REQUIRE_THAT(
       group2_received, Catch::Matchers::UnorderedEquals(group2_expected));
 
@@ -747,8 +736,16 @@ TEST_CASE_METHOD(
   rc = tiledb_group_open(ctx_, group1, TILEDB_WRITE);
   REQUIRE(rc == TILEDB_OK);
 
-  rc = tiledb_group_add_member(ctx_, group1, array1_uri.c_str(), false, "one");
-  REQUIRE(rc == TILEDB_OK);
+  bool add_with_type = GENERATE(true, false);
+  if (add_with_type) {
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, array1_uri.c_str(), false, "one", TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+  } else {
+    rc =
+        tiledb_group_add_member(ctx_, group1, array1_uri.c_str(), false, "one");
+    REQUIRE(rc == TILEDB_OK);
+  }
 
   // Close group from write mode
   rc = tiledb_group_close(ctx_, group1);
@@ -782,8 +779,15 @@ TEST_CASE_METHOD(
   rc = tiledb_group_remove_member(ctx_, group1, "one");
   REQUIRE(rc == TILEDB_OK);
   // Add one back with different URI
-  rc = tiledb_group_add_member(ctx_, group1, array2_uri.c_str(), false, "one");
-  REQUIRE(rc == TILEDB_OK);
+  if (add_with_type) {
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, array2_uri.c_str(), false, "one", TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+  } else {
+    rc =
+        tiledb_group_add_member(ctx_, group1, array2_uri.c_str(), false, "one");
+    REQUIRE(rc == TILEDB_OK);
+  }
 
   // Close group
   rc = tiledb_group_close(ctx_, group1);
@@ -854,15 +858,34 @@ TEST_CASE_METHOD(
   rc = tiledb_group_open(ctx_, group2, TILEDB_WRITE);
   REQUIRE(rc == TILEDB_OK);
 
-  rc = tiledb_group_add_member(ctx_, group1, array1_uri.c_str(), false, "one");
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_group_add_member(ctx_, group1, array2_uri.c_str(), false, "two");
-  REQUIRE(rc == TILEDB_OK);
-  rc =
-      tiledb_group_add_member(ctx_, group2, array3_uri.c_str(), false, "three");
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_group_add_member(ctx_, group1, group2_uri.c_str(), false, "four");
-  REQUIRE(rc == TILEDB_OK);
+  bool add_with_type = GENERATE(true, false);
+  if (add_with_type) {
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, array1_uri.c_str(), false, "one", TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, array2_uri.c_str(), false, "two", TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group2, array3_uri.c_str(), false, "three", TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, group2_uri.c_str(), false, "four", TILEDB_GROUP);
+    REQUIRE(rc == TILEDB_OK);
+  } else {
+    rc =
+        tiledb_group_add_member(ctx_, group1, array1_uri.c_str(), false, "one");
+    REQUIRE(rc == TILEDB_OK);
+    rc =
+        tiledb_group_add_member(ctx_, group1, array2_uri.c_str(), false, "two");
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group2, array3_uri.c_str(), false, "three");
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group1, group2_uri.c_str(), false, "four");
+    REQUIRE(rc == TILEDB_OK);
+  }
 
   // Close group from write mode
   rc = tiledb_group_close(ctx_, group1);
@@ -1213,18 +1236,34 @@ TEST_CASE_METHOD(
   rc = tiledb_group_open(ctx_, group2, TILEDB_WRITE);
   REQUIRE(rc == TILEDB_OK);
 
-  rc = tiledb_group_add_member(
-      ctx_, group1, array1_relative_uri.c_str(), true, nullptr);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_group_add_member(
-      ctx_, group1, array2_relative_uri.c_str(), true, nullptr);
-  REQUIRE(rc == TILEDB_OK);
-  rc = tiledb_group_add_member(
-      ctx_, group2, array3_relative_uri.c_str(), true, nullptr);
-  REQUIRE(rc == TILEDB_OK);
-  rc =
-      tiledb_group_add_member(ctx_, group1, group2_uri.c_str(), false, nullptr);
-  REQUIRE(rc == TILEDB_OK);
+  bool add_with_type = GENERATE(true, false);
+  if (add_with_type) {
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, array1_relative_uri.c_str(), true, nullptr, TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, array2_relative_uri.c_str(), true, nullptr, TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group2, array3_relative_uri.c_str(), true, nullptr, TILEDB_ARRAY);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member_with_type(
+        ctx_, group1, group2_uri.c_str(), false, nullptr, TILEDB_GROUP);
+    REQUIRE(rc == TILEDB_OK);
+  } else {
+    rc = tiledb_group_add_member(
+        ctx_, group1, array1_relative_uri.c_str(), true, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group1, array2_relative_uri.c_str(), true, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group2, array3_relative_uri.c_str(), true, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+    rc = tiledb_group_add_member(
+        ctx_, group1, group2_uri.c_str(), false, nullptr);
+    REQUIRE(rc == TILEDB_OK);
+  }
 
   // Close group from write mode
   rc = tiledb_group_close(ctx_, group1);
@@ -1490,16 +1529,18 @@ TEST_CASE_METHOD(
   remove_temp_dir(temp_dir);
 }
 
-TEST_CASE_METHOD(GroupFx, "C API: Group, dump", "[capi][group][dump]") {
+TEST_CASE_METHOD(
+    GroupFx,
+    "C API: Group, dump",
+    "[capi][group][dump][rest-fails][sc-48099]") {
   // Create and open group in write mode
-  // TODO: refactor for each supported FS.
   std::string temp_dir = fs_vec_[0]->temp_dir();
   create_temp_dir(temp_dir);
 
-  tiledb::sm::URI group1_uri(temp_dir + "group1");
+  std::string group1_uri = vfs_array_uri(fs_vec_[0], temp_dir + "group1");
   REQUIRE(tiledb_group_create(ctx_, group1_uri.c_str()) == TILEDB_OK);
 
-  tiledb::sm::URI group2_uri(temp_dir + "group2");
+  std::string group2_uri = vfs_array_uri(fs_vec_[0], temp_dir + "group2");
   REQUIRE(tiledb_group_create(ctx_, group2_uri.c_str()) == TILEDB_OK);
 
   tiledb_group_t* group1;
@@ -1522,14 +1563,36 @@ TEST_CASE_METHOD(GroupFx, "C API: Group, dump", "[capi][group][dump]") {
   rc = tiledb_group_open(ctx_, group1, TILEDB_READ);
   REQUIRE(rc == TILEDB_OK);
 
-  char* group_dump;
-  rc = tiledb_group_dump_str(ctx_, group1, &group_dump, 1);
-  REQUIRE(rc == TILEDB_OK);
-  REQUIRE(group_dump != nullptr);
-  REQUIRE(
-      std::string(group_dump) ==
-      "group1 GROUP\n|-- group2 GROUP (does not exist)\n");
-  free(group_dump);
+  std::string group_dump;
+  // Test both the new and the deprecated API. Remove the else part when the
+  // deprecated API gets removed.
+  bool use_dump_v2 = GENERATE(true, false);
+  if (use_dump_v2) {
+    tiledb_string_t* group_dump_handle;
+    rc = tiledb_group_dump_str_v2(ctx_, group1, &group_dump_handle, 1);
+    REQUIRE(rc == TILEDB_OK);
+    REQUIRE(group_dump_handle != nullptr);
+    const char* group_dump_ptr;
+    size_t group_dump_size;
+    rc = tiledb_string_view(
+        group_dump_handle, &group_dump_ptr, &group_dump_size);
+    REQUIRE(rc == TILEDB_OK);
+    REQUIRE(group_dump_ptr != nullptr);
+    group_dump = std::string(group_dump_ptr, group_dump_size);
+    rc = tiledb_string_free(&group_dump_handle);
+    REQUIRE(rc == TILEDB_OK);
+  } else {
+    char* group_dump_ptr;
+    rc = tiledb_group_dump_str(ctx_, group1, &group_dump_ptr, 1);
+    REQUIRE(rc == TILEDB_OK);
+    REQUIRE(group_dump_ptr != nullptr);
+    group_dump = std::string(group_dump_ptr);
+    // Usually it's not safe to call free() on this pointer, but since we are
+    // building everything at the same time, tiledb's and tiledb_unit's
+    // allocators are the same.
+    free(group_dump_ptr);
+  }
+  REQUIRE(group_dump == "group1 GROUP\n|-- group2 GROUP (does not exist)\n");
 
   rc = tiledb_group_close(ctx_, group1);
   REQUIRE(rc == TILEDB_OK);
