@@ -51,7 +51,7 @@ struct ConsolidationWithTimestampsFx {
   // TileDB context.
   Context ctx_;
   VFS vfs_;
-  sm::StorageManager* sm_;
+  sm::ContextResources* resources_;
 
   // Constructors/destructors.
   ConsolidationWithTimestampsFx();
@@ -60,13 +60,11 @@ struct ConsolidationWithTimestampsFx {
   // Functions.
   void set_legacy();
   void create_sparse_array(bool allows_dups = false);
-  void create_sparse_array_v11();
   void write_sparse(
       std::vector<int> a1,
       std::vector<uint64_t> dim1,
       std::vector<uint64_t> dim2,
       uint64_t timestamp);
-  void write_sparse_v11(uint64_t timestamp);
   void consolidate_sparse(bool vacuum = false);
   void consolidate_sparse(
       uint64_t start_time, uint64_t end_time, bool vacuum = false);
@@ -99,7 +97,7 @@ ConsolidationWithTimestampsFx::ConsolidationWithTimestampsFx()
   Config config;
   config.set("sm.consolidation.buffer_size", "1000");
   ctx_ = Context(config);
-  sm_ = ctx_.ptr().get()->storage_manager();
+  resources_ = &ctx_.ptr().get()->resources();
   vfs_ = VFS(ctx_);
 }
 
@@ -113,7 +111,7 @@ void ConsolidationWithTimestampsFx::set_legacy() {
   config.set("sm.query.sparse_unordered_with_dups.reader", "legacy");
 
   ctx_ = Context(config);
-  sm_ = ctx_.ptr().get()->storage_manager();
+  resources_ = &ctx_.ptr().get()->resources();
   vfs_ = VFS(ctx_);
 }
 
@@ -149,25 +147,17 @@ void ConsolidationWithTimestampsFx::create_sparse_array(bool allows_dups) {
   Array::create(SPARSE_ARRAY_NAME, schema);
 }
 
-void ConsolidationWithTimestampsFx::create_sparse_array_v11() {
-  // Get the v11 sparse array.
-  std::string v11_arrays_dir =
-      std::string(TILEDB_TEST_INPUTS_DIR) + "/arrays/sparse_array_v11";
-  REQUIRE(
-      tiledb_vfs_copy_dir(
-          ctx_.ptr().get(),
-          vfs_.ptr().get(),
-          v11_arrays_dir.c_str(),
-          SPARSE_ARRAY_NAME) == TILEDB_OK);
-}
-
 void ConsolidationWithTimestampsFx::write_sparse(
     std::vector<int> a1,
     std::vector<uint64_t> dim1,
     std::vector<uint64_t> dim2,
     uint64_t timestamp) {
   // Open array.
-  Array array(ctx_, SPARSE_ARRAY_NAME, TILEDB_WRITE, timestamp);
+  Array array(
+      ctx_,
+      SPARSE_ARRAY_NAME,
+      TILEDB_WRITE,
+      tiledb::TemporalPolicy(tiledb::TimeTravel, timestamp));
 
   // Create query.
   Query query(ctx_, array, TILEDB_WRITE);
@@ -183,36 +173,6 @@ void ConsolidationWithTimestampsFx::write_sparse(
   array.close();
 }
 
-void ConsolidationWithTimestampsFx::write_sparse_v11(uint64_t timestamp) {
-  // Prepare cell buffers.
-  std::vector<int> buffer_a1{0, 1, 2, 3};
-  std::vector<uint64_t> buffer_a2{0, 1, 3, 6};
-  std::string buffer_var_a2("abbcccdddd");
-  std::vector<float> buffer_a3{0.1f, 0.2f, 1.1f, 1.2f, 2.1f, 2.2f, 3.1f, 3.2f};
-  std::vector<uint64_t> buffer_coords_dim1{1, 1, 1, 2};
-  std::vector<uint64_t> buffer_coords_dim2{1, 2, 4, 3};
-
-  // Open array.
-  Array array(ctx_, SPARSE_ARRAY_NAME, TILEDB_WRITE, timestamp);
-
-  // Create query.
-  Query query(ctx_, array, TILEDB_WRITE);
-  query.set_layout(TILEDB_GLOBAL_ORDER);
-  query.set_data_buffer("a1", buffer_a1);
-  query.set_data_buffer(
-      "a2", (void*)buffer_var_a2.c_str(), buffer_var_a2.size());
-  query.set_offsets_buffer("a2", buffer_a2);
-  query.set_data_buffer("a3", buffer_a3);
-  query.set_data_buffer("d1", buffer_coords_dim1);
-  query.set_data_buffer("d2", buffer_coords_dim2);
-
-  // Submit query
-  query.submit_and_finalize();
-
-  // Close array.
-  array.close();
-}
-
 void ConsolidationWithTimestampsFx::consolidate_sparse(bool vacuum) {
   auto config = ctx_.config();
   Array::consolidate(ctx_, SPARSE_ARRAY_NAME, &config);
@@ -290,7 +250,11 @@ void ConsolidationWithTimestampsFx::read_sparse(
     uint64_t timestamp,
     std::vector<uint64_t>* timestamps_ptr) {
   // Open array.
-  Array array(ctx_, SPARSE_ARRAY_NAME, TILEDB_READ, timestamp);
+  Array array(
+      ctx_,
+      SPARSE_ARRAY_NAME,
+      TILEDB_READ,
+      tiledb::TemporalPolicy(tiledb::TimeTravel, timestamp));
 
   // Create query.
   Query query(ctx_, array, TILEDB_READ);
@@ -419,7 +383,7 @@ TEST_CASE_METHOD(
   };
 
   for (auto& ts : test_timestamps) {
-    sm::ArrayDirectory array_dir(sm_->resources(), array_uri, ts[0], ts[1]);
+    sm::ArrayDirectory array_dir(*resources_, array_uri, ts[0], ts[1]);
 
     // Check that only the consolidated fragment is visible.
     auto filtered_fragment_uris = array_dir.filtered_fragment_uris(false);
@@ -437,7 +401,7 @@ TEST_CASE_METHOD(
   };
 
   for (auto& ts : test_timestamps) {
-    sm::ArrayDirectory array_dir(sm_->resources(), array_uri, ts[0], ts[1]);
+    sm::ArrayDirectory array_dir(*resources_, array_uri, ts[0], ts[1]);
 
     // Check that no fragment is visible
     auto filtered_fragment_uris = array_dir.filtered_fragment_uris(false);
@@ -505,12 +469,12 @@ TEST_CASE_METHOD(
   }
 
   remove_sparse_array();
-  create_sparse_array_v11();
+  test::create_sparse_array_v11(ctx_.ptr().get(), SPARSE_ARRAY_NAME);
   // Write first fragment.
-  write_sparse_v11(1);
+  test::write_sparse_v11(ctx_.ptr().get(), SPARSE_ARRAY_NAME, 1);
 
   // Write second fragment.
-  write_sparse_v11(3);
+  test::write_sparse_v11(ctx_.ptr().get(), SPARSE_ARRAY_NAME, 3);
 
   // Consolidate.
   consolidate_sparse();
@@ -525,7 +489,7 @@ TEST_CASE_METHOD(
   };
 
   for (auto& ts : test_timestamps) {
-    sm::ArrayDirectory array_dir(sm_->resources(), array_uri, ts[0], ts[1]);
+    sm::ArrayDirectory array_dir(*resources_, array_uri, ts[0], ts[1]);
 
     // Check that only the first fragment is visible on an old array.
     auto filtered_fragment_uris = array_dir.filtered_fragment_uris(false);
@@ -543,7 +507,7 @@ TEST_CASE_METHOD(
   };
 
   for (auto& ts : test_timestamps) {
-    sm::ArrayDirectory array_dir(sm_->resources(), array_uri, ts[0], ts[1]);
+    sm::ArrayDirectory array_dir(*resources_, array_uri, ts[0], ts[1]);
 
     // Check that no fragment is visible
     auto filtered_fragment_uris = array_dir.filtered_fragment_uris(false);
@@ -672,8 +636,8 @@ TEST_CASE_METHOD(
 
   // Will only allow to load two tiles out of 3.
   Config cfg;
-  cfg.set("sm.mem.total_budget", "11000");
-  cfg.set("sm.mem.reader.sparse_global_order.ratio_coords", "0.4");
+  cfg.set("sm.mem.total_budget", "30000");
+  cfg.set("sm.mem.reader.sparse_global_order.ratio_coords", "0.15");
   ctx_ = Context(cfg);
 
   std::string stats;
@@ -688,8 +652,7 @@ TEST_CASE_METHOD(
 
   // Make sure there was an internal loop on the reader.
   CHECK(
-      stats.find(
-          "\"Context.StorageManager.Query.Reader.internal_loop_num\": 2") !=
+      stats.find("\"Context.Query.Reader.internal_loop_num\": 2") !=
       std::string::npos);
 
   remove_sparse_array();
@@ -722,8 +685,8 @@ TEST_CASE_METHOD(
 
   // Will only allow to load two tiles out of 3.
   Config cfg;
-  cfg.set("sm.mem.total_budget", "11000");
-  cfg.set("sm.mem.reader.sparse_global_order.ratio_coords", "0.4");
+  cfg.set("sm.mem.total_budget", "30000");
+  cfg.set("sm.mem.reader.sparse_global_order.ratio_coords", "0.15");
   ctx_ = Context(cfg);
 
   std::string stats;
@@ -741,8 +704,7 @@ TEST_CASE_METHOD(
 
   // Make sure there was an internal loop on the reader.
   CHECK(
-      stats.find(
-          "\"Context.StorageManager.Query.Reader.internal_loop_num\": 2") !=
+      stats.find("\"Context.Query.Reader.internal_loop_num\": 2") !=
       std::string::npos);
 
   remove_sparse_array();
@@ -1270,9 +1232,11 @@ TEST_CASE_METHOD(
   // fully removed for overlapping ranges, this test case can be deleted.
   tiledb::Config cfg;
   cfg.set("sm.merge_overlapping_ranges_experimental", "false");
-  cfg["sm.consolidation.total_buffer_size"] = "1048576";
+  cfg["sm.mem.consolidation.buffers_weight"] = "1";
+  cfg["sm.mem.consolidation.reader_weight"] = "5000";
+  cfg["sm.mem.consolidation.writer_weight"] = "5000";
   ctx_ = Context(cfg);
-  sm_ = ctx_.ptr().get()->storage_manager();
+  resources_ = &ctx_.ptr().get()->resources();
   vfs_ = VFS(ctx_);
 
   // Write first fragment.
@@ -1309,8 +1273,11 @@ TEST_CASE_METHOD(
   query.set_data_buffer(tiledb_timestamps(), timestamps);
 
   // Add overlapping ranges
-  query.add_range<uint64_t>(1, 2, 3);
-  query.add_range<uint64_t>(1, 2, 3);
+  Subarray subarray(ctx_, array);
+  subarray.add_range<uint64_t>(1, 2, 3);
+  subarray.add_range<uint64_t>(1, 2, 3);
+  subarray.set_config(cfg);
+  query.set_subarray(subarray);
 
   // Submit/finalize the query
   query.submit();
@@ -1507,8 +1474,7 @@ TEST_CASE_METHOD(
     // partially overlaps with the first 1:3 consolidated fragment.
     read_sparse(a, dim1, dim2, stats, layout, 2);
     CHECK(
-        stats.find(
-            "\"Context.StorageManager.Query.Reader.num_tiles_read\": 4") !=
+        stats.find("\"Context.Query.Reader.num_tiles_read\": 4") !=
         std::string::npos);
 
     // Same but skip timestamps: 4 - 1 = 3, since 0:4 array fully overlaps with
@@ -1516,8 +1482,7 @@ TEST_CASE_METHOD(
     // cells.
     read_sparse(a, dim1, dim2, stats, layout, 4);
     CHECK(
-        stats.find(
-            "\"Context.StorageManager.Query.Reader.num_tiles_read\": 3") !=
+        stats.find("\"Context.Query.Reader.num_tiles_read\": 3") !=
         std::string::npos);
 
     // Read 2 tiles for dim1, dim2, attr a1 and 1 for timestamps: 3 * 2 + 1 = 7
@@ -1525,24 +1490,21 @@ TEST_CASE_METHOD(
     // overlaps with the second 5:7 consolidated fragment.
     read_sparse(a, dim1, dim2, stats, layout, 6);
     CHECK(
-        stats.find(
-            "\"Context.StorageManager.Query.Reader.num_tiles_read\": 7") !=
+        stats.find("\"Context.Query.Reader.num_tiles_read\": 7") !=
         std::string::npos);
 
     // Read 2 tiles for dim1, dim2, attr a1 and skip timestamps: 3 * 2, since
     // 0:8 array fully overlaps with both consolidateded fragments 1:3 and 5:7.
     read_sparse(a, dim1, dim2, stats, layout, 8);
     CHECK(
-        stats.find(
-            "\"Context.StorageManager.Query.Reader.num_tiles_read\": 6") !=
+        stats.find("\"Context.Query.Reader.num_tiles_read\": 6") !=
         std::string::npos);
 
     // Read 2 tiles for dim1, dim2, attr a1 and timestamps: 4 * 2, since 3:5
     // array partially overlaps with both consolidateded fragments 1:3 and 5:7.
     reopen_sparse(a, dim1, dim2, stats, layout, 3, 5);
     CHECK(
-        stats.find(
-            "\"Context.StorageManager.Query.Reader.num_tiles_read\": 8") !=
+        stats.find("\"Context.Query.Reader.num_tiles_read\": 8") !=
         std::string::npos);
   }
 
@@ -1550,8 +1512,7 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> timestamps(16);
     read_sparse(a, dim1, dim2, stats, layout, 4, &timestamps);
     CHECK(
-        stats.find(
-            "\"Context.StorageManager.Query.Reader.num_tiles_read\": 4") !=
+        stats.find("\"Context.Query.Reader.num_tiles_read\": 4") !=
         std::string::npos);
   }
 
@@ -1569,8 +1530,7 @@ TEST_CASE_METHOD(
     consolidate_sparse(vacuum);
     read_sparse(a, dim1, dim2, stats, layout, 4);
     CHECK(
-        stats.find(
-            "\"Context.StorageManager.Query.Reader.num_tiles_read\": 4") !=
+        stats.find("\"Context.Query.Reader.num_tiles_read\": 4") !=
         std::string::npos);
   }
 
@@ -1588,8 +1548,7 @@ TEST_CASE_METHOD(
     // have any.
     read_sparse(a, dim1, dim2, stats, layout, 2);
     CHECK(
-        stats.find(
-            "\"Context.StorageManager.Query.Reader.num_tiles_read\": 3") !=
+        stats.find("\"Context.Query.Reader.num_tiles_read\": 3") !=
         std::string::npos);
 
     // Request timestamps - expect no timestamps getting read since fragment
@@ -1597,8 +1556,7 @@ TEST_CASE_METHOD(
     std::vector<uint64_t> timestamps(16);
     read_sparse(a, dim1, dim2, stats, layout, 4, &timestamps);
     CHECK(
-        stats.find(
-            "\"Context.StorageManager.Query.Reader.num_tiles_read\": 6") !=
+        stats.find("\"Context.Query.Reader.num_tiles_read\": 6") !=
         std::string::npos);
   }
 
