@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2023 TileDB Inc.
+ * @copyright Copyright (c) 2023-2024 TileDB Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -34,12 +34,15 @@
 #include "test/support/src/ast_helpers.h"
 #include "test/support/src/helpers.h"
 #include "test/support/src/vfs_helpers.h"
+#include "tiledb/api/c_api/array/array_api_internal.h"
 #include "tiledb/api/c_api/context/context_api_internal.h"
 #include "tiledb/common/stdx_string.h"
 #include "tiledb/sm/array/array_directory.h"
+#include "tiledb/sm/array/array_operations.h"
 #include "tiledb/sm/c_api/tiledb_struct_def.h"
-#include "tiledb/sm/cpp_api/group_experimental.h"
+#include "tiledb/sm/cpp_api/group.h"
 #include "tiledb/sm/cpp_api/tiledb"
+#include "tiledb/sm/enums/encryption_type.h"
 
 #ifdef _WIN32
 #include "tiledb/sm/filesystem/win.h"
@@ -58,7 +61,6 @@ struct DeletesFx {
   VFSTestSetup vfs_test_setup_;
   Context ctx_;
   VFS vfs_;
-  sm::StorageManager* sm_;
 
   const std::string array_name_;
   // to be used for direct VFS operations in [rest] testcases where array_name_
@@ -67,7 +69,8 @@ struct DeletesFx {
   const std::string group_name_;
 
   std::string key_ = "0123456789abcdeF0123456789abcdeF";
-  const tiledb_encryption_type_t enc_type_ = TILEDB_AES_256_GCM;
+  std::string enc_type_str_ =
+      encryption_type_str((tiledb::sm::EncryptionType)TILEDB_AES_256_GCM);
 
   // Constructors/destructors.
   DeletesFx();
@@ -78,14 +81,12 @@ struct DeletesFx {
   void create_dir(const std::string& path);
   void create_simple_array(const std::string& path);
   void create_sparse_array(bool allows_dups = false, bool encrypt = false);
-  void create_sparse_array_v11();
   void write_sparse(
       std::vector<int> a1,
       std::vector<uint64_t> dim1,
       std::vector<uint64_t> dim2,
       uint64_t timestamp,
       bool encrypt = false);
-  void write_sparse_v11(uint64_t timestamp);
   std::vector<tiledb::Object> read_group(const tiledb::Group& group) const;
   void read_sparse(
       std::vector<int>& a1,
@@ -124,7 +125,6 @@ DeletesFx::DeletesFx()
   vfs_test_setup_.update_config(config.ptr().get());
   ctx_ = vfs_test_setup_.ctx();
   vfs_ = VFS(ctx_);
-  sm_ = ctx_.ptr().get()->storage_manager();
 }
 
 void DeletesFx::set_purge_deleted_cells() {
@@ -133,6 +133,7 @@ void DeletesFx::set_purge_deleted_cells() {
   config.set("sm.consolidation.purge_deleted_cells", "true");
   vfs_test_setup_.update_config(config.ptr().get());
   ctx_ = vfs_test_setup_.ctx();
+  vfs_ = VFS(ctx_);
 }
 
 void DeletesFx::set_legacy() {
@@ -142,6 +143,7 @@ void DeletesFx::set_legacy() {
   config.set("sm.query.sparse_unordered_with_dups.reader", "legacy");
   vfs_test_setup_.update_config(config.ptr().get());
   ctx_ = vfs_test_setup_.ctx();
+  vfs_ = VFS(ctx_);
 }
 
 void DeletesFx::create_dir(const std::string& path) {
@@ -168,6 +170,17 @@ void DeletesFx::create_simple_array(const std::string& path) {
 }
 
 void DeletesFx::create_sparse_array(bool allows_dups, bool encrypt) {
+  Config config;
+  config.set("sm.consolidation.buffer_size", "1000");
+  if (encrypt) {
+    config["sm.encryption_type"] = enc_type_str_.c_str();
+    config["sm.encryption_key"] = key_;
+  }
+
+  vfs_test_setup_.update_config(config.ptr().get());
+  ctx_ = vfs_test_setup_.ctx();
+  vfs_ = VFS(ctx_);
+
   // Create dimensions.
   auto d1 = Dimension::create<uint64_t>(ctx_, "d1", {{1, 4}}, 2);
   auto d2 = Dimension::create<uint64_t>(ctx_, "d2", {{1, 4}}, 2);
@@ -196,11 +209,7 @@ void DeletesFx::create_sparse_array(bool allows_dups, bool encrypt) {
   filter_list.add_filter(filter);
   schema.set_coords_filter_list(filter_list);
 
-  if (encrypt) {
-    Array::create(array_name_.c_str(), schema, enc_type_, key_);
-  } else {
-    Array::create(array_name_.c_str(), schema);
-  }
+  Array::create(array_name_.c_str(), schema);
 }
 
 void DeletesFx::write_sparse(
@@ -240,48 +249,6 @@ void DeletesFx::write_sparse(
   array->close();
 }
 
-void DeletesFx::create_sparse_array_v11() {
-  // Get the v11 sparse array.
-  std::string v11_arrays_dir =
-      std::string(TILEDB_TEST_INPUTS_DIR) + "/arrays/sparse_array_v11";
-  REQUIRE(
-      tiledb_vfs_copy_dir(
-          ctx_.ptr().get(),
-          vfs_.ptr().get(),
-          v11_arrays_dir.c_str(),
-          array_name_.c_str()) == TILEDB_OK);
-}
-
-void DeletesFx::write_sparse_v11(uint64_t timestamp) {
-  // Prepare cell buffers.
-  std::vector<int> buffer_a1{0, 1, 2, 3};
-  std::vector<uint64_t> buffer_a2{0, 1, 3, 6};
-  std::string buffer_var_a2("abbcccdddd");
-  std::vector<float> buffer_a3{0.1f, 0.2f, 1.1f, 1.2f, 2.1f, 2.2f, 3.1f, 3.2f};
-  std::vector<uint64_t> buffer_coords_dim1{1, 1, 1, 2};
-  std::vector<uint64_t> buffer_coords_dim2{1, 2, 4, 3};
-
-  // Open array.
-  Array array(ctx_, array_name_.c_str(), TILEDB_WRITE, timestamp);
-
-  // Create query.
-  Query query(ctx_, array, TILEDB_WRITE);
-  query.set_layout(TILEDB_GLOBAL_ORDER);
-  query.set_data_buffer("a1", buffer_a1);
-  query.set_data_buffer(
-      "a2", (void*)buffer_var_a2.c_str(), buffer_var_a2.size());
-  query.set_offsets_buffer("a2", buffer_a2);
-  query.set_data_buffer("a3", buffer_a3);
-  query.set_data_buffer("d1", buffer_coords_dim1);
-  query.set_data_buffer("d2", buffer_coords_dim2);
-
-  // Submit/finalize the query.
-  query.submit_and_finalize();
-
-  // Close array.
-  array.close();
-}
-
 std::vector<tiledb::Object> DeletesFx::read_group(
     const tiledb::Group& group) const {
   std::vector<tiledb::Object> ret;
@@ -420,18 +387,17 @@ void DeletesFx::check_delete_conditions(
         TILEDB_READ,
         TemporalPolicy(TimeTravel, timestamp));
   }
-  auto array_ptr = array->ptr()->array_;
+  auto array_ptr = array->ptr();
 
   // Load delete conditions.
-  auto&& [st, delete_conditions, update_values] =
-      sm_->load_delete_and_update_conditions(*(array_ptr->opened_array()));
-  REQUIRE(st.ok());
-  REQUIRE(delete_conditions->size() == qcs.size());
+  auto&& [delete_conditions, update_values] = load_delete_and_update_conditions(
+      ctx_.ptr().get()->resources(), *array_ptr->opened_array().get());
+  REQUIRE(delete_conditions.size() == qcs.size());
 
   for (uint64_t i = 0; i < qcs.size(); i++) {
     // Compare to negated condition.
     auto cmp = qcs[i].ptr()->query_condition_->negated_condition();
-    CHECK(tiledb::test::ast_equal(delete_conditions->at(i).ast(), cmp.ast()));
+    CHECK(tiledb::test::ast_equal(delete_conditions.at(i).ast(), cmp.ast()));
   }
 
   array->close();
@@ -1648,12 +1614,12 @@ TEST_CASE_METHOD(
   }
 
   remove_sparse_array();
-  create_sparse_array_v11();
+  test::create_sparse_array_v11(ctx_.ptr().get(), array_name_);
   // Write first fragment.
-  write_sparse_v11(1);
+  test::write_sparse_v11(ctx_.ptr().get(), array_name_, 1);
 
   // Write second fragment.
-  write_sparse_v11(3);
+  test::write_sparse_v11(ctx_.ptr().get(), array_name_, 3);
 
   // Consolidate.
   consolidate_sparse();
@@ -1693,7 +1659,7 @@ TEST_CASE_METHOD(
   const char* extraneous_fragments[1] = {extraneous_fragment.c_str()};
   REQUIRE_THROWS_WITH(
       Array::delete_fragments_list(ctx_, array_name_, extraneous_fragments, 1),
-      Catch::Matchers::ContainsSubstring("Failed to delete fragments_list"));
+      Catch::Matchers::ContainsSubstring("Failed to delete fragments list"));
   CHECK(tiledb::test::num_fragments(ctx_, array_name_) == 2);
 
   remove_sparse_array();
@@ -1934,8 +1900,8 @@ TEST_CASE_METHOD(
   auto array_name = array_name_ + "/";
 
   // Write to v11 array
-  create_sparse_array_v11();
-  write_sparse_v11(1);
+  test::create_sparse_array_v11(ctx_.ptr().get(), array_name_);
+  test::write_sparse_v11(ctx_.ptr().get(), array_name_, 1);
   std::string extraneous_file_path = array_name + "extraneous_file";
   vfs_.touch(extraneous_file_path);
 
