@@ -384,9 +384,12 @@ TEST_CASE(
     query_r.set_data_buffer("d1", r_buff_d1);
     query_r.set_data_buffer("d2", r_buff_d2);
 
-    query_r.add_range("d1", (int32_t)1, (int32_t)5);
-    query_r.add_range("d2", (int32_t)1, (int32_t)3);
-    query_r.add_range("d2", (int32_t)2, (int32_t)4);
+    Subarray subarray_r(ctx, array_r);
+    subarray_r.add_range("d1", (int32_t)1, (int32_t)5);
+    subarray_r.add_range("d2", (int32_t)1, (int32_t)3);
+    subarray_r.add_range("d2", (int32_t)2, (int32_t)4);
+    subarray_r.set_config(cfg);
+    query_r.set_subarray(subarray_r);
     query_r.set_layout(TILEDB_UNORDERED);
     // Submit query
     query_r.submit();
@@ -1778,7 +1781,9 @@ TEST_CASE(
   Config config;
   config["sm.consolidation.mode"] = "fragments";
   config["sm.vacuum.mode"] = "fragments";
-  config["sm.consolidation.total_buffer_size"] = "1048576";
+  config["sm.mem.consolidation.buffers_weight"] = "1";
+  config["sm.mem.consolidation.reader_weight"] = "5000";
+  config["sm.mem.consolidation.writer_weight"] = "5000";
   CHECK_NOTHROW(Array::consolidate(ctx, array_name, &config));
   CHECK_NOTHROW(Array::vacuum(ctx, array_name, &config));
   auto contents = vfs.ls(get_fragment_dir(array_name));
@@ -1874,8 +1879,10 @@ TEST_CASE(
     query_r.set_data_buffer("d2", r_buff_d2);
     query_r.set_offsets_buffer("d2", r_off_d2);
     query_r.set_layout(TILEDB_ROW_MAJOR);
-    query_r.add_range(0, std::string("3"), std::string("z"));
-    query_r.add_range(1, std::string("a"), std::string("vase"));
+    Subarray subarray_r(ctx, array_r);
+    subarray_r.add_range(0, std::string("3"), std::string("z"));
+    subarray_r.add_range(1, std::string("a"), std::string("vase"));
+    query_r.set_subarray(subarray_r);
     CHECK_NOTHROW(query_r.submit());
 
     // Check results
@@ -1915,8 +1922,10 @@ TEST_CASE(
     query_r.set_data_buffer("d2", r_buff_d2);
     query_r.set_offsets_buffer("d2", r_off_d2);
     query_r.set_layout(TILEDB_GLOBAL_ORDER);
-    query_r.add_range(0, std::string("3"), std::string("z"));
-    query_r.add_range(1, std::string("a"), std::string("vase"));
+    Subarray subarray_r(ctx, array_r);
+    subarray_r.add_range(0, std::string("3"), std::string("z"));
+    subarray_r.add_range(1, std::string("a"), std::string("vase"));
+    query_r.set_subarray(subarray_r);
     CHECK_NOTHROW(query_r.submit());
 
     // Check results. Hilbert values:
@@ -2087,8 +2096,10 @@ TEST_CASE(
     query_r.set_offsets_buffer("d1", r_off_d1);
     query_r.set_data_buffer("d2", r_buff_d2);
     query_r.set_offsets_buffer("d2", r_off_d2);
-    query_r.add_range(0, std::string("1a", 2), std::string("w", 1));
-    query_r.add_range(1, std::string("ca", 2), std::string("t1", 2));
+    Subarray subarray_r(ctx, array_r);
+    subarray_r.add_range(0, std::string("1a", 2), std::string("w", 1));
+    subarray_r.add_range(1, std::string("ca", 2), std::string("t1", 2));
+    query_r.set_subarray(subarray_r);
     query_r.set_layout(TILEDB_GLOBAL_ORDER);
     CHECK_NOTHROW(query_r.submit());
 
