@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2018-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2018-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,7 @@
 
 #include <test/support/tdb_catch.h>
 #include "test/support/src/helpers.h"
-#include "tiledb/sm/c_api/tiledb_struct_def.h"
+#include "tiledb/api/c_api/array/array_api_internal.h"
 #include "tiledb/sm/cpp_api/tiledb"
 #include "tiledb/sm/query/readers/aggregators/tile_metadata.h"
 #include "tiledb/sm/tile/tile_metadata_generator.h"
@@ -283,19 +283,21 @@ struct CPPFixedTileMetadataFx {
     CHECK(rc == TILEDB_OK);
 
     // Load fragment metadata.
-    auto frag_meta = array->array_->fragment_metadata();
-    auto& enc_key = array->array_->get_encryption_key();
-    frag_meta[f]->load_fragment_min_max_sum_null_count(enc_key);
+    auto frag_meta = array->fragment_metadata();
+    auto& enc_key = array->get_encryption_key();
+    frag_meta[f]->loaded_metadata()->load_fragment_min_max_sum_null_count(
+        enc_key);
 
     // Load the metadata and validate coords netadata.
     bool has_coords = layout != TILEDB_ROW_MAJOR;
     if (has_coords) {
       std::vector<std::string> names{"d"};
-      frag_meta[f]->load_rtree(enc_key);
-      frag_meta[f]->load_tile_min_values(enc_key, names);
-      frag_meta[f]->load_tile_max_values(enc_key, names);
-      frag_meta[f]->load_tile_sum_values(enc_key, names);
-      frag_meta[f]->load_tile_null_count_values(enc_key, names);
+      frag_meta[f]->loaded_metadata()->load_rtree(enc_key);
+      frag_meta[f]->loaded_metadata()->load_tile_min_values(enc_key, names);
+      frag_meta[f]->loaded_metadata()->load_tile_max_values(enc_key, names);
+      frag_meta[f]->loaded_metadata()->load_tile_sum_values(enc_key, names);
+      frag_meta[f]->loaded_metadata()->load_tile_null_count_values(
+          enc_key, names);
 
       // Validation.
       // Min/max/sum for all null tile are invalid.
@@ -307,18 +309,18 @@ struct CPPFixedTileMetadataFx {
 
           // Validate no min.
           CHECK_THROWS_WITH(
-              frag_meta[f]->get_min("d"),
+              frag_meta[f]->loaded_metadata()->get_min("d"),
               "FragmentMetadata: Trying to access fragment min metadata that's "
               "not present");
 
           // Validate no max.
           CHECK_THROWS_WITH(
-              frag_meta[f]->get_max("d"),
+              frag_meta[f]->loaded_metadata()->get_max("d"),
               "FragmentMetadata: Trying to access fragment max metadata that's "
               "not present");
 
           // Validate sum.
-          auto sum = frag_meta[f]->get_sum("d");
+          auto sum = frag_meta[f]->loaded_metadata()->get_sum("d");
           CHECK(*(int64_t*)sum == correct_sum);
         }
 
@@ -341,7 +343,8 @@ struct CPPFixedTileMetadataFx {
               "present");
 
           // Validate sum.
-          auto sum = frag_meta[f]->get_tile_sum("d", tile_idx);
+          auto sum =
+              frag_meta[f]->loaded_metadata()->get_tile_sum("d", tile_idx);
           CHECK(*(int64_t*)sum == correct_sum);
 
           // Validate the tile metadata structure.
@@ -359,19 +362,19 @@ struct CPPFixedTileMetadataFx {
       if constexpr (std::is_same<TestType, std::byte>::value) {
         // Validate no min.
         CHECK_THROWS_WITH(
-            frag_meta[f]->get_min("a"),
+            frag_meta[f]->loaded_metadata()->get_min("a"),
             "FragmentMetadata: Trying to access fragment min metadata that's "
             "not present");
 
         // Validate no max.
         CHECK_THROWS_WITH(
-            frag_meta[f]->get_max("a"),
+            frag_meta[f]->loaded_metadata()->get_max("a"),
             "FragmentMetadata: Trying to access fragment max metadata that's "
             "not present");
 
         // Validate no sum.
         CHECK_THROWS_WITH(
-            frag_meta[f]->get_sum("a"),
+            frag_meta[f]->loaded_metadata()->get_sum("a"),
             "FragmentMetadata: Trying to access fragment sum metadata that's "
             "not present");
       } else {
@@ -379,7 +382,7 @@ struct CPPFixedTileMetadataFx {
         if (!all_null) {
           if constexpr (std::is_same<TestType, char>::value) {
             // Validate min.
-            auto& min = frag_meta[f]->get_min("a");
+            auto& min = frag_meta[f]->loaded_metadata()->get_min("a");
             CHECK(min.size() == cell_val_num);
 
             // For strings, the index is stored in a signed value, switch to
@@ -393,7 +396,7 @@ struct CPPFixedTileMetadataFx {
                          cell_val_num));
 
             // Validate max.
-            auto& max = frag_meta[f]->get_max("a");
+            auto& max = frag_meta[f]->loaded_metadata()->get_max("a");
             CHECK(max.size() == cell_val_num);
 
             // For strings, the index is stored in a signed value, switch to
@@ -408,23 +411,23 @@ struct CPPFixedTileMetadataFx {
 
             // Validate no sum.
             CHECK_THROWS_WITH(
-                frag_meta[f]->get_sum("a"),
+                frag_meta[f]->loaded_metadata()->get_sum("a"),
                 "FragmentMetadata: Trying to access fragment sum metadata "
                 "that's not present");
           } else {
             // Validate min.
-            auto& min = frag_meta[f]->get_min("a");
+            auto& min = frag_meta[f]->loaded_metadata()->get_min("a");
             CHECK(min.size() == sizeof(TestType));
             CHECK(0 == memcmp(min.data(), &correct_mins_[f], min.size()));
 
             // Validate max.
-            auto& max = frag_meta[f]->get_max("a");
+            auto& max = frag_meta[f]->loaded_metadata()->get_max("a");
             CHECK(max.size() == sizeof(TestType));
             CHECK(0 == memcmp(max.data(), &correct_maxs_[f], max.size()));
 
             if constexpr (!std::is_same<TestType, unsigned char>::value) {
               // Validate sum.
-              auto sum = frag_meta[f]->get_sum("a");
+              auto sum = frag_meta[f]->loaded_metadata()->get_sum("a");
               if constexpr (std::is_integral_v<TestType>) {
                 CHECK(*(int64_t*)sum == correct_sums_int_[f]);
               } else {
@@ -437,11 +440,11 @@ struct CPPFixedTileMetadataFx {
 
       // Check null count.
       if (nullable) {
-        auto nc = frag_meta[f]->get_null_count("a");
+        auto nc = frag_meta[f]->loaded_metadata()->get_null_count("a");
         CHECK(nc == correct_null_counts_[f]);
       } else {
         CHECK_THROWS_WITH(
-            frag_meta[f]->get_null_count("a"),
+            frag_meta[f]->loaded_metadata()->get_null_count("a"),
             "FragmentMetadata: Trying to access fragment null count metadata "
             "that's not present");
       }
@@ -449,10 +452,11 @@ struct CPPFixedTileMetadataFx {
 
     // Load attribute metadata.
     std::vector<std::string> names{"a"};
-    frag_meta[f]->load_tile_min_values(enc_key, names);
-    frag_meta[f]->load_tile_max_values(enc_key, names);
-    frag_meta[f]->load_tile_sum_values(enc_key, names);
-    frag_meta[f]->load_tile_null_count_values(enc_key, names);
+    frag_meta[f]->loaded_metadata()->load_tile_min_values(enc_key, names);
+    frag_meta[f]->loaded_metadata()->load_tile_max_values(enc_key, names);
+    frag_meta[f]->loaded_metadata()->load_tile_sum_values(enc_key, names);
+    frag_meta[f]->loaded_metadata()->load_tile_null_count_values(
+        enc_key, names);
 
     // Validate attribute metadta.
     // Min/max/sum for all null tile are invalid.
@@ -471,7 +475,7 @@ struct CPPFixedTileMetadataFx {
 
       // Validate no sum.
       CHECK_THROWS_WITH(
-          frag_meta[f]->get_tile_sum("a", 0),
+          frag_meta[f]->loaded_metadata()->get_tile_sum("a", 0),
           "FragmentMetadata: Trying to access tile sum metadata that's not "
           "present");
     } else {
@@ -508,7 +512,7 @@ struct CPPFixedTileMetadataFx {
 
             // Validate no sum.
             CHECK_THROWS_WITH(
-                frag_meta[f]->get_tile_sum("a", tile_idx),
+                frag_meta[f]->loaded_metadata()->get_tile_sum("a", tile_idx),
                 "FragmentMetadata: Trying to access tile sum metadata that's "
                 "not "
                 "present");
@@ -554,7 +558,8 @@ struct CPPFixedTileMetadataFx {
 
             if constexpr (!std::is_same<TestType, unsigned char>::value) {
               // Validate sum.
-              auto sum = frag_meta[f]->get_tile_sum("a", tile_idx);
+              auto sum =
+                  frag_meta[f]->loaded_metadata()->get_tile_sum("a", tile_idx);
               if constexpr (std::is_integral_v<TestType>) {
                 CHECK(*(int64_t*)sum == correct_tile_sums_int_[f][tile_idx]);
                 CHECK(
@@ -575,11 +580,12 @@ struct CPPFixedTileMetadataFx {
     // Check null count.
     for (uint64_t tile_idx = 0; tile_idx < num_tiles_; tile_idx++) {
       if (nullable) {
-        auto nc = frag_meta[f]->get_tile_null_count("a", tile_idx);
+        auto nc =
+            frag_meta[f]->loaded_metadata()->get_tile_null_count("a", tile_idx);
         CHECK(nc == correct_tile_null_counts_[f][tile_idx]);
       } else {
         CHECK_THROWS_WITH(
-            frag_meta[f]->get_tile_null_count("a", tile_idx),
+            frag_meta[f]->loaded_metadata()->get_tile_null_count("a", tile_idx),
             "FragmentMetadata: Trying to access tile null count metadata "
             "that's not present");
       }
@@ -800,7 +806,7 @@ struct CPPVarTileMetadataFx {
       query.set_data_buffer("d", d);
     }
     query.set_offsets_buffer("a", a_offsets);
-    query.set_buffer("a", a_var);
+    query.set_data_buffer("a", a_var);
 
     if (nullable) {
       query.set_validity_buffer("a", a_val);
@@ -828,19 +834,21 @@ struct CPPVarTileMetadataFx {
     CHECK(rc == TILEDB_OK);
 
     // Load fragment metadata.
-    auto frag_meta = array->array_->fragment_metadata();
-    auto& enc_key = array->array_->get_encryption_key();
-    frag_meta[f]->load_fragment_min_max_sum_null_count(enc_key);
+    auto frag_meta = array->fragment_metadata();
+    auto& enc_key = array->get_encryption_key();
+    frag_meta[f]->loaded_metadata()->load_fragment_min_max_sum_null_count(
+        enc_key);
 
     // Load the metadata and validate coords netadata.
     bool has_coords = layout != TILEDB_ROW_MAJOR;
     if (has_coords) {
       std::vector<std::string> names{"d"};
-      frag_meta[f]->load_rtree(enc_key);
-      frag_meta[f]->load_tile_min_values(enc_key, names);
-      frag_meta[f]->load_tile_max_values(enc_key, names);
-      frag_meta[f]->load_tile_sum_values(enc_key, names);
-      frag_meta[f]->load_tile_null_count_values(enc_key, names);
+      frag_meta[f]->loaded_metadata()->load_rtree(enc_key);
+      frag_meta[f]->loaded_metadata()->load_tile_min_values(enc_key, names);
+      frag_meta[f]->loaded_metadata()->load_tile_max_values(enc_key, names);
+      frag_meta[f]->loaded_metadata()->load_tile_sum_values(enc_key, names);
+      frag_meta[f]->loaded_metadata()->load_tile_null_count_values(
+          enc_key, names);
 
       // Validation.
       // Min/max/sum for all null tile are invalid.
@@ -852,18 +860,18 @@ struct CPPVarTileMetadataFx {
 
           // Validate no min.
           CHECK_THROWS_WITH(
-              frag_meta[f]->get_min("d"),
+              frag_meta[f]->loaded_metadata()->get_min("d"),
               "FragmentMetadata: Trying to access fragment min metadata that's "
               "not present");
 
           // Validate no max.
           CHECK_THROWS_WITH(
-              frag_meta[f]->get_max("d"),
+              frag_meta[f]->loaded_metadata()->get_max("d"),
               "FragmentMetadata: Trying to access fragment max metadata that's "
               "not present");
 
           // Validate sum.
-          auto sum = frag_meta[f]->get_sum("d");
+          auto sum = frag_meta[f]->loaded_metadata()->get_sum("d");
           CHECK(*(int64_t*)sum == correct_sum);
         }
 
@@ -886,7 +894,8 @@ struct CPPVarTileMetadataFx {
               "present");
 
           // Validate sum.
-          auto sum = frag_meta[f]->get_tile_sum("d", tile_idx);
+          auto sum =
+              frag_meta[f]->loaded_metadata()->get_tile_sum("d", tile_idx);
           CHECK(*(int64_t*)sum == correct_sum);
 
           // Validate the tile metadata structure.
@@ -903,7 +912,7 @@ struct CPPVarTileMetadataFx {
       // Min/max/sum for all null tile are invalid.
       if (!all_null) {
         // Validate min.
-        auto& min = frag_meta[f]->get_min("a");
+        auto& min = frag_meta[f]->loaded_metadata()->get_min("a");
         CHECK(min.size() == strings_[correct_mins_[f]].size());
         CHECK(
             0 == strncmp(
@@ -912,7 +921,7 @@ struct CPPVarTileMetadataFx {
                      strings_[correct_mins_[f]].size()));
 
         // Validate max.
-        auto& max = frag_meta[f]->get_max("a");
+        auto& max = frag_meta[f]->loaded_metadata()->get_max("a");
         CHECK(max.size() == strings_[correct_maxs_[f]].size());
         CHECK(
             0 == strncmp(
@@ -922,18 +931,18 @@ struct CPPVarTileMetadataFx {
 
         // Validate no sum.
         CHECK_THROWS_WITH(
-            frag_meta[f]->get_sum("a"),
+            frag_meta[f]->loaded_metadata()->get_sum("a"),
             "FragmentMetadata: Trying to access fragment sum metadata that's "
             "not present");
       }
 
       // Check null count.
       if (nullable) {
-        auto nc = frag_meta[f]->get_null_count("a");
+        auto nc = frag_meta[f]->loaded_metadata()->get_null_count("a");
         CHECK(nc == correct_null_counts_[f]);
       } else {
         CHECK_THROWS_WITH(
-            frag_meta[f]->get_null_count("a"),
+            frag_meta[f]->loaded_metadata()->get_null_count("a"),
             "FragmentMetadata: Trying to access fragment null count metadata "
             "that's not present");
       }
@@ -941,10 +950,11 @@ struct CPPVarTileMetadataFx {
 
     // Load attribute metadata.
     std::vector<std::string> names{"a"};
-    frag_meta[f]->load_tile_min_values(enc_key, names);
-    frag_meta[f]->load_tile_max_values(enc_key, names);
-    frag_meta[f]->load_tile_sum_values(enc_key, names);
-    frag_meta[f]->load_tile_null_count_values(enc_key, names);
+    frag_meta[f]->loaded_metadata()->load_tile_min_values(enc_key, names);
+    frag_meta[f]->loaded_metadata()->load_tile_max_values(enc_key, names);
+    frag_meta[f]->loaded_metadata()->load_tile_sum_values(enc_key, names);
+    frag_meta[f]->loaded_metadata()->load_tile_null_count_values(
+        enc_key, names);
 
     // Validate attribute metadata.
     // Min/max/sum for all null tile are invalid.
@@ -974,7 +984,7 @@ struct CPPVarTileMetadataFx {
 
         // Validate no sum.
         CHECK_THROWS_WITH(
-            frag_meta[f]->get_tile_sum("a", tile_idx),
+            frag_meta[f]->loaded_metadata()->get_tile_sum("a", tile_idx),
             "FragmentMetadata: Trying to access tile sum metadata that's not "
             "present");
 
@@ -988,11 +998,12 @@ struct CPPVarTileMetadataFx {
     // Check null count.
     for (uint64_t tile_idx = 0; tile_idx < num_tiles_; tile_idx++) {
       if (nullable) {
-        auto nc = frag_meta[f]->get_tile_null_count("a", tile_idx);
+        auto nc =
+            frag_meta[f]->loaded_metadata()->get_tile_null_count("a", tile_idx);
         CHECK(nc == correct_tile_null_counts_[f][tile_idx]);
       } else {
         CHECK_THROWS_WITH(
-            frag_meta[f]->get_tile_null_count("a", tile_idx),
+            frag_meta[f]->loaded_metadata()->get_tile_null_count("a", tile_idx),
             "FragmentMetadata: Trying to access tile null count metadata "
             "that's not present");
       }
@@ -1155,38 +1166,40 @@ struct CPPFixedTileMetadataPartialFx {
     CHECK(rc == TILEDB_OK);
 
     // Load fragment metadata.
-    auto frag_meta = array->array_->fragment_metadata();
-    auto& enc_key = array->array_->get_encryption_key();
-    frag_meta[0]->load_fragment_min_max_sum_null_count(enc_key);
+    auto frag_meta = array->fragment_metadata();
+    auto& enc_key = array->get_encryption_key();
+    frag_meta[0]->loaded_metadata()->load_fragment_min_max_sum_null_count(
+        enc_key);
 
     // Do fragment metadata first for attribute.
     {
       // Validate min.
-      auto& min = frag_meta[0]->get_min("a");
+      auto& min = frag_meta[0]->loaded_metadata()->get_min("a");
       CHECK(min.size() == sizeof(double));
 
       double correct_min = 1.1;
       CHECK(0 == memcmp(min.data(), &correct_min, min.size()));
 
       // Validate max.
-      auto& max = frag_meta[0]->get_max("a");
+      auto& max = frag_meta[0]->loaded_metadata()->get_max("a");
       CHECK(max.size() == sizeof(double));
 
       double correct_max = 4.9;
       CHECK(0 == memcmp(max.data(), &correct_max, max.size()));
 
       // Validate sum.
-      auto sum = frag_meta[0]->get_sum("a");
+      auto sum = frag_meta[0]->loaded_metadata()->get_sum("a");
       double correct_sum = 46.7;
       CHECK(*(double*)sum - correct_sum < 0.0001);
     }
 
     // Load attribute metadata.
     std::vector<std::string> names{"a"};
-    frag_meta[0]->load_tile_min_values(enc_key, names);
-    frag_meta[0]->load_tile_max_values(enc_key, names);
-    frag_meta[0]->load_tile_sum_values(enc_key, names);
-    frag_meta[0]->load_tile_null_count_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_min_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_max_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_sum_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_null_count_values(
+        enc_key, names);
 
     std::vector<double> correct_tile_mins{1.1, 2.1, 3.2, 4.1};
     std::vector<double> correct_tile_maxs{1.7, 2.6, 3.8, 4.9};
@@ -1203,7 +1216,7 @@ struct CPPFixedTileMetadataPartialFx {
       CHECK(0 == memcmp(&max, &correct_tile_maxs[tile_idx], sizeof(double)));
 
       // Validate sum.
-      auto sum = frag_meta[0]->get_tile_sum("a", tile_idx);
+      auto sum = frag_meta[0]->loaded_metadata()->get_tile_sum("a", tile_idx);
       CHECK(*(double*)sum - correct_tile_sums[tile_idx] < 0.0001);
 
       // Validate the tile metadata structure.
@@ -1333,20 +1346,21 @@ struct CPPVarTileMetadataPartialFx {
     CHECK(rc == TILEDB_OK);
 
     // Load fragment metadata.
-    auto frag_meta = array->array_->fragment_metadata();
-    auto& enc_key = array->array_->get_encryption_key();
-    frag_meta[0]->load_fragment_min_max_sum_null_count(enc_key);
+    auto frag_meta = array->fragment_metadata();
+    auto& enc_key = array->get_encryption_key();
+    frag_meta[0]->loaded_metadata()->load_fragment_min_max_sum_null_count(
+        enc_key);
 
     // Do fragment metadata first for attribute.
     {
       // Validate min.
-      auto& min = frag_meta[0]->get_min("a");
+      auto& min = frag_meta[0]->loaded_metadata()->get_min("a");
       std::string correct_min = "1.1";
       CHECK(min.size() == correct_min.size());
       CHECK(0 == memcmp(min.data(), correct_min.data(), min.size()));
 
       // Validate max.
-      auto& max = frag_meta[0]->get_max("a");
+      auto& max = frag_meta[0]->loaded_metadata()->get_max("a");
       std::string correct_max = "4.9";
       CHECK(max.size() == correct_max.size());
       CHECK(0 == memcmp(max.data(), correct_max.data(), max.size()));
@@ -1354,10 +1368,11 @@ struct CPPVarTileMetadataPartialFx {
 
     // Load attribute metadata.
     std::vector<std::string> names{"a"};
-    frag_meta[0]->load_tile_min_values(enc_key, names);
-    frag_meta[0]->load_tile_max_values(enc_key, names);
-    frag_meta[0]->load_tile_sum_values(enc_key, names);
-    frag_meta[0]->load_tile_null_count_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_min_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_max_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_sum_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_null_count_values(
+        enc_key, names);
 
     std::vector<std::string> correct_tile_mins{"1.1", "2.1", "3.2", "4.1"};
     std::vector<std::string> correct_tile_maxs{"1.7", "2.6", "3.8", "4.9"};
@@ -1499,50 +1514,52 @@ struct CPPTileMetadataStringDimFx {
     CHECK(rc == TILEDB_OK);
 
     // Load fragment metadata.
-    auto frag_meta = array->array_->fragment_metadata();
-    auto& enc_key = array->array_->get_encryption_key();
-    frag_meta[0]->load_fragment_min_max_sum_null_count(enc_key);
+    auto frag_meta = array->fragment_metadata();
+    auto& enc_key = array->get_encryption_key();
+    frag_meta[0]->loaded_metadata()->load_fragment_min_max_sum_null_count(
+        enc_key);
 
     // Do fragment metadata first.
     {
       // Validate mins.
-      auto& min = frag_meta[0]->get_min("a");
+      auto& min = frag_meta[0]->loaded_metadata()->get_min("a");
       CHECK(min.size() == sizeof(double));
       CHECK(*static_cast<double*>(static_cast<void*>(min.data())) == 4);
 
       CHECK_THROWS_WITH(
-          frag_meta[0]->get_min("d1"),
+          frag_meta[0]->loaded_metadata()->get_min("d1"),
           "FragmentMetadata: Trying to access fragment min metadata that's "
           "not present");
 
       CHECK_THROWS_WITH(
-          frag_meta[0]->get_min("d2"),
+          frag_meta[0]->loaded_metadata()->get_min("d2"),
           "FragmentMetadata: Trying to access fragment min metadata that's "
           "not present");
 
       // Validate maxs.
-      auto& max = frag_meta[0]->get_max("a");
+      auto& max = frag_meta[0]->loaded_metadata()->get_max("a");
       CHECK(max.size() == sizeof(double));
       CHECK(*static_cast<double*>(static_cast<void*>(max.data())) == 7);
 
       CHECK_THROWS_WITH(
-          frag_meta[0]->get_max("d1"),
+          frag_meta[0]->loaded_metadata()->get_max("d1"),
           "FragmentMetadata: Trying to access fragment max metadata that's "
           "not present");
 
       CHECK_THROWS_WITH(
-          frag_meta[0]->get_max("d2"),
+          frag_meta[0]->loaded_metadata()->get_max("d2"),
           "FragmentMetadata: Trying to access fragment max metadata that's "
           "not present");
     }
 
     // Load metadata.
     std::vector<std::string> names{"a", "d1", "d2"};
-    frag_meta[0]->load_rtree(enc_key);
-    frag_meta[0]->load_tile_min_values(enc_key, names);
-    frag_meta[0]->load_tile_max_values(enc_key, names);
-    frag_meta[0]->load_tile_sum_values(enc_key, names);
-    frag_meta[0]->load_tile_null_count_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_rtree(enc_key);
+    frag_meta[0]->loaded_metadata()->load_tile_min_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_max_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_sum_values(enc_key, names);
+    frag_meta[0]->loaded_metadata()->load_tile_null_count_values(
+        enc_key, names);
 
     // Validate min.
     CHECK(frag_meta[0]->get_tile_min_as<double>("a", 0) == 4);
@@ -1567,7 +1584,8 @@ struct CPPTileMetadataStringDimFx {
         "present");
 
     // Validate sum.
-    CHECK(*(double*)frag_meta[0]->get_tile_sum("a", 0) == 22);
+    CHECK(
+        *(double*)frag_meta[0]->loaded_metadata()->get_tile_sum("a", 0) == 22);
 
     // Validate the tile metadata structure.
     auto full_tile_data_a = frag_meta[0]->get_tile_metadata("a", 0);
