@@ -37,6 +37,7 @@
 #include "tiledb/common/logger.h"
 #include "tiledb/common/memory_tracker.h"
 #include "tiledb/sm/array_schema/attribute.h"
+#include "tiledb/sm/array_schema/current_domain.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/array_schema/dimension_label.h"
 #include "tiledb/sm/array_schema/domain.h"
@@ -52,10 +53,7 @@
 #include "tiledb/sm/filter/webp_filter.h"
 #include "tiledb/sm/fragment/fragment_identifier.h"
 #include "tiledb/sm/misc/hilbert.h"
-#include "tiledb/sm/misc/integral_type_casts.h"
 #include "tiledb/sm/misc/tdb_time.h"
-#include "tiledb/sm/storage_manager/context_resources.h"
-#include "tiledb/sm/tile/generic_tile_io.h"
 #include "tiledb/storage_format/uri/generate_uri.h"
 #include "tiledb/type/apply_with_type.h"
 
@@ -105,7 +103,9 @@ ArraySchema::ArraySchema(
           memory_tracker_->get_resource(MemoryType::DIMENSION_LABELS))
     , enumeration_map_(memory_tracker_->get_resource(MemoryType::ENUMERATION))
     , enumeration_path_map_(
-          memory_tracker_->get_resource(MemoryType::ENUMERATION_PATHS)) {
+          memory_tracker_->get_resource(MemoryType::ENUMERATION_PATHS))
+    , current_domain_(make_shared<CurrentDomain>(
+          memory_tracker, constants::current_domain_version)) {
   // Set up default filter pipelines for coords, offsets, and validity values.
   coords_filters_.add_filter(CompressionFilter(
       constants::coords_compression,
@@ -142,6 +142,7 @@ ArraySchema::ArraySchema(
     FilterPipeline cell_var_offsets_filters,
     FilterPipeline cell_validity_filters,
     FilterPipeline coords_filters,
+    shared_ptr<CurrentDomain> current_domain,
     shared_ptr<MemoryTracker> memory_tracker)
     : memory_tracker_(memory_tracker)
     , uri_(uri)
@@ -166,7 +167,8 @@ ArraySchema::ArraySchema(
           memory_tracker_->get_resource(MemoryType::ENUMERATION_PATHS))
     , cell_var_offsets_filters_(cell_var_offsets_filters)
     , cell_validity_filters_(cell_validity_filters)
-    , coords_filters_(coords_filters) {
+    , coords_filters_(coords_filters)
+    , current_domain_(current_domain) {
   for (auto atr : attributes) {
     attributes_.push_back(atr);
   }
@@ -185,8 +187,8 @@ ArraySchema::ArraySchema(
     dim_map_[dim->name()] = dim;
   }
 
-  for (auto& [enmr_name, enmr_uri] : enumeration_path_map_) {
-    (void)enmr_uri;
+  for (auto& [enmr_name, enmr_filename] : enumeration_path_map_) {
+    (void)enmr_filename;
     enumeration_map_[enmr_name] = nullptr;
   }
 
@@ -257,6 +259,7 @@ ArraySchema::ArraySchema(const ArraySchema& array_schema)
     , cell_var_offsets_filters_{array_schema.cell_var_offsets_filters_}
     , cell_validity_filters_{array_schema.cell_validity_filters_}
     , coords_filters_{array_schema.coords_filters_}
+    , current_domain_(array_schema.current_domain_)
     , mtx_{} {
   throw_if_not_ok(set_domain(array_schema.domain_));
 
@@ -311,11 +314,6 @@ shared_ptr<const Attribute> ArraySchema::shared_attribute(
   return attributes_[it->second.index];
 }
 
-const tdb::pmr::vector<shared_ptr<const Attribute>>& ArraySchema::attributes()
-    const {
-  return attributes_;
-}
-
 uint64_t ArraySchema::capacity() const {
   return capacity_;
 }
@@ -689,57 +687,6 @@ ArraySchema::dimension_size_type ArraySchema::dim_num() const {
   return domain_->dim_num();
 }
 
-void ArraySchema::dump(FILE* out) const {
-  if (out == nullptr)
-    out = stdout;
-
-  std::stringstream ss;
-  ss << "- Array type: " << array_type_str(array_type_) << "\n";
-  ss << "- Cell order: " << layout_str(cell_order_) << "\n";
-  ss << "- Tile order: " << layout_str(tile_order_) << "\n";
-  ss << "- Capacity: " << capacity_ << "\n";
-  ss << "- Allows duplicates: " << (allows_dups_ ? "true" : "false") << "\n";
-  ss << "- Coordinates filters: " << coords_filters_.size();
-  fprintf(out, "%s", ss.str().c_str());
-
-  coords_filters_.dump(out);
-  fprintf(
-      out,
-      "\n- Offsets filters: %u",
-      (unsigned)cell_var_offsets_filters_.size());
-  cell_var_offsets_filters_.dump(out);
-  fprintf(
-      out, "\n- Validity filters: %u", (unsigned)cell_validity_filters_.size());
-  cell_validity_filters_.dump(out);
-  fprintf(out, "\n");
-
-  if (domain_ != nullptr)
-    domain_->dump(out);
-
-  for (auto& attr : attributes_) {
-    fprintf(out, "\n");
-    attr->dump(out);
-  }
-
-  for (auto& enmr_iter : enumeration_map_) {
-    fprintf(out, "\n");
-    if (enmr_iter.second != nullptr) {
-      enmr_iter.second->dump(out);
-    } else {
-      std::stringstream ss;
-      ss << "### Enumeration ###" << std::endl;
-      ss << "- Name: " << enmr_iter.first << std::endl;
-      ss << "- Loaded: false" << std::endl;
-      fprintf(out, "%s", ss.str().c_str());
-    }
-  }
-
-  for (auto& label : dimension_labels_) {
-    fprintf(out, "\n");
-    label->dump(out);
-  }
-}
-
 Status ArraySchema::has_attribute(
     const std::string& name, bool* has_attr) const {
   *has_attr = false;
@@ -786,94 +733,6 @@ bool ArraySchema::is_nullable(const std::string& name) const {
   return attr->nullable();
 }
 
-// ===== FORMAT =====
-// version (uint32_t)
-// allow_dups (bool)
-// array_type (uint8_t)
-// tile_order (uint8_t)
-// cell_order (uint8_t)
-// capacity (uint64_t)
-// coords_filters (see FilterPipeline::serialize)
-// cell_var_offsets_filters (see FilterPipeline::serialize)
-// cell_validity_filters (see FilterPipeline::serialize)
-// domain
-// attribute_num (uint32_t)
-//   attribute #1
-//   attribute #2
-//   ...
-// dimension_label_num (uint32_t)
-//   dimension_label #1
-//   dimension_label #2
-//   ...
-void ArraySchema::serialize(Serializer& serializer) const {
-  // Write version, which is always the current version. Despite
-  // the in-memory `version_`, we will serialize every array schema
-  // as the latest version.
-  const format_version_t version = constants::format_version;
-  serializer.write<format_version_t>(version);
-
-  // Write allows_dups
-  serializer.write<uint8_t>(allows_dups_);
-
-  // Write array type
-  auto array_type = (uint8_t)array_type_;
-  serializer.write<uint8_t>(array_type);
-
-  // Write tile and cell order
-  auto tile_order = (uint8_t)tile_order_;
-  serializer.write<uint8_t>(tile_order);
-  auto cell_order = (uint8_t)cell_order_;
-  serializer.write<uint8_t>(cell_order);
-
-  // Write capacity
-  serializer.write<uint64_t>(capacity_);
-
-  // Write coords filters
-  coords_filters_.serialize(serializer);
-
-  // Write offsets filters
-  cell_var_offsets_filters_.serialize(serializer);
-
-  // Write validity filters
-  cell_validity_filters_.serialize(serializer);
-
-  // Write domain
-  domain_->serialize(serializer, version);
-
-  // Write attributes
-  auto attribute_num = (uint32_t)attributes_.size();
-  serializer.write<uint32_t>(attribute_num);
-  for (auto& attr : attributes_) {
-    attr->serialize(serializer, version);
-  }
-
-  // Write dimension labels
-  auto label_num = static_cast<uint32_t>(dimension_labels_.size());
-  if (label_num != dimension_labels_.size()) {
-    throw ArraySchemaException(
-        "Overflow when attempting to serialize label number.");
-  }
-  serializer.write<uint32_t>(label_num);
-  for (auto& label : dimension_labels_) {
-    label->serialize(serializer, version);
-  }
-
-  // Write Enumeration path map
-  auto enmr_num =
-      utils::safe_integral_cast<size_t, uint32_t>(enumeration_map_.size());
-
-  serializer.write<uint32_t>(enmr_num);
-  for (auto& [enmr_name, enmr_uri] : enumeration_path_map_) {
-    auto enmr_name_size = static_cast<uint32_t>(enmr_name.size());
-    serializer.write<uint32_t>(enmr_name_size);
-    serializer.write(enmr_name.data(), enmr_name_size);
-
-    auto enmr_uri_size = static_cast<uint32_t>(enmr_uri.size());
-    serializer.write<uint32_t>(enmr_uri_size);
-    serializer.write(enmr_uri.data(), enmr_uri_size);
-  }
-}
-
 Layout ArraySchema::tile_order() const {
   return tile_order_;
 }
@@ -1231,6 +1090,17 @@ std::vector<std::string> ArraySchema::get_loaded_enumeration_names() const {
   return enmr_names;
 }
 
+std::vector<shared_ptr<const Enumeration>>
+ArraySchema::get_loaded_enumerations() const {
+  std::vector<shared_ptr<const Enumeration>> enmrs;
+  for (auto& entry : enumeration_map_) {
+    if (entry.second != nullptr) {
+      enmrs.emplace_back(entry.second);
+    }
+  }
+  return enmrs;
+}
+
 bool ArraySchema::is_enumeration_loaded(
     const std::string& enumeration_name) const {
   auto iter = enumeration_map_.find(enumeration_name);
@@ -1291,7 +1161,7 @@ void ArraySchema::drop_enumeration(const std::string& enmr_name) {
     }
     if (attr_enmr_name.value() == enmr_name) {
       throw ArraySchemaException(
-          "Unable to drop enumeration '" + enmr_name + "' as it is used by " +
+          "Unable to drop enumeration '" + enmr_name + "' as it is used by" +
           " attribute '" + attr->name() + "'.");
     }
   }
@@ -1415,14 +1285,23 @@ shared_ptr<ArraySchema> ArraySchema::deserialize(
       std::string enmr_name(
           deserializer.get_ptr<char>(enmr_name_size), enmr_name_size);
 
-      auto enmr_path_size = deserializer.read<uint32_t>();
-      std::string enmr_path_name(
-          deserializer.get_ptr<char>(enmr_path_size), enmr_path_size);
+      auto enmr_filename_size = deserializer.read<uint32_t>();
+      std::string enmr_filename(
+          deserializer.get_ptr<char>(enmr_filename_size), enmr_filename_size);
 
-      enumeration_path_map[enmr_name] = enmr_path_name;
+      enumeration_path_map[enmr_name] = enmr_filename;
     }
   }
 
+  // Load the array current domain, if this is an older array, it'll get by
+  // default an empty current domain object
+  auto current_domain = make_shared<CurrentDomain>(
+      memory_tracker, constants::current_domain_version);
+  if (version >= constants::current_domain_min_format_version) {
+    current_domain =
+        CurrentDomain::deserialize(deserializer, memory_tracker, domain);
+  }
+
   // Validate
   if (cell_order == Layout::HILBERT &&
       domain->dim_num() > Hilbert::HC_MAX_DIM) {
@@ -1472,6 +1351,7 @@ shared_ptr<ArraySchema> ArraySchema::deserialize(
       FilterPipeline(
           coords_filters,
           version < 5 ? domain->dimension_ptr(0)->type() : Datatype::UINT64),
+      current_domain,
       memory_tracker);
 }
 
@@ -1524,6 +1404,11 @@ Status ArraySchema::set_cell_order(Layout cell_order) {
         Status_ArraySchemaError("Cannot set cell order; Hilbert order is only "
                                 "applicable to sparse arrays"));
 
+  if (cell_order == Layout::UNORDERED)
+    return LOG_STATUS(Status_ArraySchemaError(
+        "Cannot set cell order; Cannot create ArraySchema "
+        "with UNORDERED cell order"));
+
   cell_order_ = cell_order;
 
   return Status::Ok();
@@ -1618,6 +1503,11 @@ Status ArraySchema::set_tile_order(Layout tile_order) {
     return LOG_STATUS(Status_ArraySchemaError(
         "Cannot set tile order; Hilbert order is not applicable to tiles"));
 
+  if (tile_order == Layout::UNORDERED)
+    return LOG_STATUS(Status_ArraySchemaError(
+        "Cannot set tile order; Cannot create ArraySchema "
+        "with UNORDERED tile order"));
+
   tile_order_ = tile_order;
   return Status::Ok();
 }
@@ -1650,6 +1540,10 @@ uint64_t ArraySchema::timestamp_start() const {
   return timestamp_range_.first;
 }
 
+uint64_t ArraySchema::timestamp_end() const {
+  return timestamp_range_.second;
+}
+
 const URI& ArraySchema::uri() const {
   return uri_;
 }
@@ -1782,4 +1676,91 @@ void ArraySchema::generate_uri(
       array_uri_.join_path(constants::array_schema_dir_name).join_path(name_);
 }
 
+void ArraySchema::expand_current_domain(
+    shared_ptr<CurrentDomain> new_current_domain) {
+  if (new_current_domain == nullptr) {
+    throw ArraySchemaException(
+        "The argument specified for current domain expansion is nullptr.");
+  }
+
+  // Check that the new current domain expands the existing one and not shrinks
+  // it. Every current domain covers an empty current domain.
+  if (!current_domain_->empty() &&
+      !current_domain_->covered(new_current_domain)) {
+    throw ArraySchemaException(
+        "The current domain of an array can only be expanded, please adjust "
+        "your new current domain object.");
+  }
+
+  new_current_domain->ndrectangle()->set_domain(this->shared_domain());
+  new_current_domain->check_schema_sanity(this->shared_domain());
+
+  current_domain_ = new_current_domain;
+}
+
+shared_ptr<CurrentDomain> ArraySchema::get_current_domain() const {
+  return current_domain_;
+}
+
+void ArraySchema::set_current_domain(shared_ptr<CurrentDomain> current_domain) {
+  if (current_domain == nullptr) {
+    throw ArraySchemaException(
+        "The argument specified for setting the current domain on the "
+        "schema is nullptr.");
+  }
+  current_domain_ = current_domain;
+}
+
 }  // namespace tiledb::sm
+
+std::ostream& operator<<(
+    std::ostream& os, const tiledb::sm::ArraySchema& schema) {
+  os << "- Array type: " << array_type_str(schema.array_type()) << std::endl;
+  os << "- Cell order: " << layout_str(schema.cell_order()) << std::endl;
+  os << "- Tile order: " << layout_str(schema.tile_order()) << std::endl;
+  os << "- Capacity: " << schema.capacity() << std::endl;
+  os << "- Allows duplicates: " << (schema.allows_dups() ? "true" : "false")
+     << std::endl;
+  os << "- Coordinates filters: " << schema.coords_filters().size();
+
+  os << schema.coords_filters();
+
+  os << std::endl
+     << "- Offsets filters: " << schema.cell_var_offsets_filters().size();
+
+  os << schema.cell_var_offsets_filters();
+  os << std::endl
+     << "- Validity filters: " << schema.cell_validity_filters().size();
+
+  os << schema.cell_validity_filters();
+  os << std::endl;
+
+  if (schema.shared_domain() != nullptr) {
+    os << *schema.shared_domain();
+  }
+
+  for (auto& attr : schema.attributes()) {
+    os << std::endl;
+    os << *attr;
+  }
+
+  for (auto& enmr_iter : schema.enumeration_map()) {
+    os << std::endl;
+    if (enmr_iter.second != nullptr) {
+      os << *enmr_iter.second;
+    } else {
+      os << "### Enumeration ###" << std::endl;
+      os << "- Name: " << enmr_iter.first << std::endl;
+      os << "- Loaded: false" << std::endl;
+    }
+  }
+
+  for (auto& label : schema.dimension_labels()) {
+    os << std::endl;
+    os << *label;
+  }
+
+  os << *schema.get_current_domain();
+
+  return os;
+}
