@@ -31,9 +31,11 @@
  */
 
 #include "domain.h"
+#include "current_domain.h"
 #include "dimension.h"
 #include "domain_data_ref.h"
 #include "domain_typed_data_view.h"
+#include "ndrectangle.h"
 #include "tiledb/common/blank.h"
 #include "tiledb/common/heap_memory.h"
 #include "tiledb/common/logger.h"
@@ -41,7 +43,6 @@
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/enums/layout.h"
 #include "tiledb/sm/misc/tdb_math.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/type/apply_with_type.h"
 #include "tiledb/type/range/range.h"
 
@@ -372,16 +373,6 @@ shared_ptr<Dimension> Domain::shared_dimension(const std::string& name) const {
   return {nullptr};
 }
 
-void Domain::dump(FILE* out) const {
-  if (out == nullptr)
-    out = stdout;
-
-  for (const auto dim : dimension_ptrs_) {
-    fprintf(out, "\n");
-    dim->dump(out);
-  }
-}
-
 void Domain::expand_ndrange(const NDRange& r1, NDRange* r2) const {
   assert(r2 != nullptr);
 
@@ -401,12 +392,13 @@ void Domain::expand_ndrange(const NDRange& r1, NDRange* r2) const {
   }
 }
 
-void Domain::expand_to_tiles(NDRange* ndrange) const {
+void Domain::expand_to_tiles_when_no_current_domain(
+    NDRange& query_ndrange) const {
   for (unsigned d = 0; d < dim_num_; ++d) {
     const auto dim = dimension_ptrs_[d];
     // Applicable only to fixed-sized dimensions
     if (!dim->var_size()) {
-      dim->expand_to_tile(&(*ndrange)[d]);
+      dim->expand_to_tile(&(query_ndrange)[d]);
     }
   }
 }
@@ -1179,3 +1171,12 @@ template uint64_t Domain::stride<float>(Layout subarray_layout) const;
 template uint64_t Domain::stride<double>(Layout subarray_layout) const;
 
 }  // namespace tiledb::sm
+
+std::ostream& operator<<(std::ostream& os, const tiledb::sm::Domain& domain) {
+  for (unsigned i = 0; i < domain.dim_num(); i++) {
+    os << std::endl;
+    os << *domain.dimension_ptr(i);
+  }
+
+  return os;
+}
