@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  * @copyright Copyright (c) 2016 MIT and Intel Corporation
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
@@ -29,8 +29,11 @@
  * @section DESCRIPTION
  *
  * This file defines the C API of TileDB for filestore.
+ *
+ * Note: These APIs are deprecated and will be removed in the next release.
  **/
 
+#include "tiledb/api/c_api/array_schema/array_schema_api_internal.h"
 #include "tiledb/api/c_api/attribute/attribute_api_internal.h"
 #include "tiledb/api/c_api/config/config_api_internal.h"
 #include "tiledb/api/c_api/dimension/dimension_api_internal.h"
@@ -85,11 +88,7 @@ int32_t tiledb_filestore_schema_create(
   if (uri) {
     // The user provided a uri, let's examine the file and get some insights
     // Get the file size, calculate a reasonable tile extent
-    tiledb::sm::VFS vfs(
-        &context.resources().stats(),
-        context.compute_tp(),
-        context.io_tp(),
-        context.storage_manager()->config());
+    auto& vfs = ctx->resources().vfs();
     uint64_t file_size;
     throw_if_not_ok(vfs.file_size(tiledb::sm::URI(uri), &file_size));
     if (file_size) {
@@ -111,7 +110,6 @@ int32_t tiledb_filestore_schema_create(
     }
   }
 
-  *array_schema = new tiledb_array_schema_t;
   try {
     // Share ownership of the internal ArraySchema ptr
     // All other calls for adding domains, attributes, etc
@@ -119,9 +117,8 @@ int32_t tiledb_filestore_schema_create(
     // the cpp objects constructed here
     auto memory_tracker = context.resources().create_memory_tracker();
     memory_tracker->set_type(sm::MemoryTrackerType::ARRAY_CREATE);
-    (*array_schema)->array_schema_ = make_shared<tiledb::sm::ArraySchema>(
-        HERE(), tiledb::sm::ArrayType::DENSE, memory_tracker);
-    auto& schema = (*array_schema)->array_schema_;
+    *array_schema = tiledb_array_schema_t::make_handle(
+        tiledb::sm::ArrayType::DENSE, memory_tracker);
 
     // Define the range of the dimension.
     uint64_t range_lo = 0;
@@ -160,13 +157,14 @@ int32_t tiledb_filestore_schema_create(
       attr->set_filter_pipeline(filter);
     }
 
-    throw_if_not_ok(schema->set_domain(domain));
-    throw_if_not_ok(schema->set_tile_order(tiledb::sm::Layout::ROW_MAJOR));
-    throw_if_not_ok(schema->set_cell_order(tiledb::sm::Layout::ROW_MAJOR));
-    throw_if_not_ok(schema->add_attribute(attr));
+    throw_if_not_ok((*array_schema)->set_domain(domain));
+    throw_if_not_ok(
+        (*array_schema)->set_tile_order(tiledb::sm::Layout::ROW_MAJOR));
+    throw_if_not_ok(
+        (*array_schema)->set_cell_order(tiledb::sm::Layout::ROW_MAJOR));
+    throw_if_not_ok((*array_schema)->add_attribute(attr));
   } catch (const std::exception& e) {
-    (*array_schema)->array_schema_ = nullptr;
-    delete *array_schema;
+    tiledb_array_schema_t::break_handle(*array_schema);
     throw api::CAPIStatusException(
         std::string("Internal TileDB uncaught exception; ") + e.what());
   }
@@ -198,11 +196,7 @@ int32_t tiledb_filestore_uri_import(
   tiledb::sm::Context& context = ctx->context();
 
   // Get the file size
-  tiledb::sm::VFS vfs(
-      &context.resources().stats(),
-      context.compute_tp(),
-      context.io_tp(),
-      context.storage_manager()->config());
+  auto& vfs = ctx->resources().vfs();
   uint64_t file_size;
   throw_if_not_ok(vfs.file_size(tiledb::sm::URI(file_uri), &file_size));
   if (!file_size) {
@@ -212,7 +206,7 @@ int32_t tiledb_filestore_uri_import(
   // Sync up the fragment timestamp and metadata timestamp
   uint64_t time_now = tiledb_timestamp_now_ms();
   auto array = make_shared<tiledb::sm::Array>(
-      HERE(), tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      HERE(), context.resources(), tiledb::sm::URI(filestore_array_uri));
   throw_if_not_ok(array->open(
       tiledb::sm::QueryType::WRITE,
       0,
@@ -271,19 +265,19 @@ int32_t tiledb_filestore_uri_import(
   // timestamped fragments in row-major order.
   bool is_tiledb_uri = array->is_remote();
   uint64_t tile_extent = compute_tile_extent_based_on_file_size(file_size);
-  auto buffer_size = get_buffer_size_from_config(
-      context.storage_manager()->config(), tile_extent);
+  auto buffer_size =
+      get_buffer_size_from_config(context.resources().config(), tile_extent);
 
-  tiledb::sm::Query query(context.storage_manager(), array);
+  tiledb::sm::Query query(
+      context.resources(),
+      context.cancellation_source(),
+      context.storage_manager(),
+      array);
   throw_if_not_ok(query.set_layout(tiledb::sm::Layout::GLOBAL_ORDER));
   std::vector<std::byte> buffer(buffer_size);
 
   tiledb::sm::Subarray subarray(
-      array.get(),
-      nullptr,
-      context.storage_manager()->logger(),
-      true,
-      context.storage_manager());
+      array.get(), nullptr, context.resources().logger(), true);
   // We need to get the right end boundary of the last space tile.
   // The last chunk either falls exactly on the end of the file
   // or it goes beyond the end of the file so that it's equal in size
@@ -300,14 +294,14 @@ int32_t tiledb_filestore_uri_import(
   query.set_subarray(subarray);
 
   auto tiledb_cloud_fix = [&](uint64_t start, uint64_t end) {
-    tiledb::sm::Query query(context.storage_manager(), array);
+    tiledb::sm::Query query(
+        context.resources(),
+        context.cancellation_source(),
+        context.storage_manager(),
+        array);
     throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
     tiledb::sm::Subarray subarray_cloud_fix(
-        array.get(),
-        nullptr,
-        context.storage_manager()->logger(),
-        true,
-        context.storage_manager());
+        array.get(), nullptr, context.resources().logger(), true);
 
     uint64_t cloud_fix_range_arr[] = {start, end};
     tiledb::type::Range subarray_range_cloud_fix(
@@ -386,11 +380,7 @@ int32_t tiledb_filestore_uri_export(
   ensure_uri_is_valid(file_uri);
 
   tiledb::sm::Context& context = ctx->context();
-  tiledb::sm::VFS vfs(
-      &context.resources().stats(),
-      context.compute_tp(),
-      context.io_tp(),
-      context.storage_manager()->config());
+  auto& vfs = ctx->resources().vfs();
   if (!vfs.open_file(tiledb::sm::URI(file_uri), tiledb::sm::VFSMode::VFS_WRITE)
            .ok()) {
     throw api::CAPIException(
@@ -399,7 +389,7 @@ int32_t tiledb_filestore_uri_export(
   }
 
   auto array = make_shared<tiledb::sm::Array>(
-      HERE(), tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      HERE(), context.resources(), tiledb::sm::URI(filestore_array_uri));
   throw_if_not_ok(array->open(
       tiledb::sm::QueryType::READ,
       tiledb::sm::EncryptionType::NO_ENCRYPTION,
@@ -422,8 +412,8 @@ int32_t tiledb_filestore_uri_export(
 
   uint64_t file_size = *static_cast<const uint64_t*>(file_size_ptr);
   uint64_t tile_extent = compute_tile_extent_based_on_file_size(file_size);
-  auto buffer_size = get_buffer_size_from_config(
-      context.storage_manager()->config(), tile_extent);
+  auto buffer_size =
+      get_buffer_size_from_config(context.resources().config(), tile_extent);
 
   std::vector<std::byte> data(buffer_size);
   uint64_t start_range = 0;
@@ -431,17 +421,17 @@ int32_t tiledb_filestore_uri_export(
   do {
     uint64_t write_size = end_range - start_range + 1;
     tiledb::sm::Subarray subarray(
-        array.get(),
-        nullptr,
-        context.storage_manager()->logger(),
-        true,
-        context.storage_manager());
+        array.get(), nullptr, context.resources().logger(), true);
     uint64_t subarray_range_arr[] = {start_range, end_range};
     tiledb::type::Range subarray_range(
         static_cast<void*>(subarray_range_arr), sizeof(uint64_t) * 2);
     subarray.add_range(0, std::move(subarray_range));
 
-    tiledb::sm::Query query(context.storage_manager(), array);
+    tiledb::sm::Query query(
+        context.resources(),
+        context.cancellation_source(),
+        context.storage_manager(),
+        array);
     throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
     query.set_subarray(subarray);
 
@@ -502,7 +492,7 @@ int32_t tiledb_filestore_buffer_import(
   // Sync up the fragment timestamp and metadata timestamp
   uint64_t time_now = tiledb_timestamp_now_ms();
   auto array = make_shared<tiledb::sm::Array>(
-      HERE(), tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      HERE(), context.resources(), tiledb::sm::URI(filestore_array_uri));
   throw_if_not_ok(array->open(
       tiledb::sm::QueryType::WRITE,
       0,
@@ -544,15 +534,15 @@ int32_t tiledb_filestore_buffer_import(
       static_cast<uint32_t>(0),
       "");
 
-  tiledb::sm::Query query(context.storage_manager(), array);
+  tiledb::sm::Query query(
+      context.resources(),
+      context.cancellation_source(),
+      context.storage_manager(),
+      array);
   throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
 
   tiledb::sm::Subarray subarray(
-      array.get(),
-      nullptr,
-      context.storage_manager()->logger(),
-      true,
-      context.storage_manager());
+      array.get(), nullptr, context.resources().logger(), true);
   uint64_t subarray_range_arr[] = {
       static_cast<uint64_t>(0), static_cast<uint64_t>(size - 1)};
   tiledb::type::Range subarray_range(
@@ -582,7 +572,7 @@ int32_t tiledb_filestore_buffer_export(
 
   tiledb::sm::Context& context = ctx->context();
   auto array = make_shared<tiledb::sm::Array>(
-      HERE(), tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      HERE(), context.resources(), tiledb::sm::URI(filestore_array_uri));
   throw_if_not_ok(array->open(
       tiledb::sm::QueryType::READ,
       tiledb::sm::EncryptionType::NO_ENCRYPTION,
@@ -610,18 +600,18 @@ int32_t tiledb_filestore_buffer_export(
   }
 
   tiledb::sm::Subarray subarray(
-      array.get(),
-      nullptr,
-      context.storage_manager()->logger(),
-      true,
-      context.storage_manager());
+      array.get(), nullptr, context.resources().logger(), true);
   uint64_t subarray_range_arr[] = {
       static_cast<uint64_t>(offset), static_cast<uint64_t>(offset + size - 1)};
   tiledb::type::Range subarray_range(
       static_cast<void*>(subarray_range_arr), sizeof(uint64_t) * 2);
   subarray.add_range(0, std::move(subarray_range));
 
-  tiledb::sm::Query query(context.storage_manager(), array);
+  tiledb::sm::Query query(
+      context.resources(),
+      context.cancellation_source(),
+      context.storage_manager(),
+      array);
   throw_if_not_ok(query.set_layout(tiledb::sm::Layout::ROW_MAJOR));
   query.set_subarray(subarray);
   uint64_t size_tmp = size;
@@ -642,7 +632,7 @@ int32_t tiledb_filestore_size(
 
   tiledb::sm::Context& context = ctx->context();
   tiledb::sm::Array array(
-      tiledb::sm::URI(filestore_array_uri), context.storage_manager());
+      context.resources(), tiledb::sm::URI(filestore_array_uri));
 
   throw_if_not_ok(array.open(
       tiledb::sm::QueryType::READ,
