@@ -115,6 +115,14 @@ class Posix : public FilesystemBase {
    */
   void remove_dir(const URI& path) const override;
 
+  /**
+   * Removes a given empty directory.
+   *
+   * @param path The path of the directory.
+   * @return true if the directory was removed, false otherwise.
+   */
+  bool remove_dir_if_empty(const std::string& path) const;
+
   /**
    * Removes a given path.
    *
@@ -260,8 +268,8 @@ class Posix : public FilesystemBase {
    * @param uri The parent path to list sub-paths.
    * @return A list of directory_entry objects
    */
-  tuple<Status, optional<std::vector<filesystem::directory_entry>>>
-  ls_with_sizes(const URI& uri) const override;
+  std::vector<filesystem::directory_entry> ls_with_sizes(
+      const URI& uri) const override;
 
   /**
    * Lists objects and object information that start with `prefix`, invoking
