@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -44,6 +44,7 @@
 #include "tiledb/common/common.h"
 #include "tiledb/common/filesystem/directory_entry.h"
 
+#include <aws/core/client/SpecifiedRetryableErrorsRetryStrategy.h>
 #include <aws/core/utils/logging/AWSLogging.h>
 #include <aws/core/utils/logging/DefaultLogSystem.h>
 #include <aws/core/utils/logging/LogLevel.h>
@@ -62,7 +63,6 @@
 #include "tiledb/sm/config/config_iter.h"
 #include "tiledb/sm/global_state/unit_test_config.h"
 #include "tiledb/sm/misc/tdb_math.h"
-#include "tiledb/sm/misc/utils.h"
 
 #ifdef _WIN32
 #if !defined(NOMINMAX)
@@ -74,6 +74,7 @@
 #endif
 
 #include "tiledb/sm/filesystem/s3.h"
+#include "tiledb/sm/filesystem/s3/AWSCredentialsProviderChain.h"
 #include "tiledb/sm/filesystem/s3/STSProfileWithWebIdentityCredentialsProvider.h"
 #include "tiledb/sm/misc/parallel_functions.h"
 
@@ -81,6 +82,13 @@ using tiledb::common::filesystem::directory_entry;
 
 namespace {
 
+/*
+ * Functions to convert strings to AWS enums.
+ *
+ * The AWS SDK provides some enum conversion functions, but they must not be
+ * used, because they have non-deterministic behavior in certain scenarios.
+ */
+
 Aws::Utils::Logging::LogLevel aws_log_name_to_level(std::string loglevel) {
   std::transform(loglevel.begin(), loglevel.end(), loglevel.begin(), ::tolower);
   if (loglevel == "fatal")
@@ -157,6 +165,47 @@ Aws::S3::Model::BucketCannedACL S3_BucketCannedACL_from_str(
     return Aws::S3::Model::BucketCannedACL::NOT_SET;
 }
 
+/**
+ * Return a S3 enum value for any recognized string or NOT_SET if
+ * B) the string is not recognized to match any of the enum values
+ *
+ * @param storage_class_str A textual string naming one of the
+ *        Aws::S3::Model::StorageClass enum members.
+ */
+Aws::S3::Model::StorageClass S3_StorageClass_from_str(
+    const std::string& storage_class_str) {
+  using Aws::S3::Model::StorageClass;
+  if (storage_class_str.empty())
+    return StorageClass::NOT_SET;
+
+  if (storage_class_str == "NOT_SET")
+    return StorageClass::NOT_SET;
+  else if (storage_class_str == "STANDARD")
+    return StorageClass::STANDARD;
+  else if (storage_class_str == "REDUCED_REDUNDANCY")
+    return StorageClass::REDUCED_REDUNDANCY;
+  else if (storage_class_str == "STANDARD_IA")
+    return StorageClass::STANDARD_IA;
+  else if (storage_class_str == "ONEZONE_IA")
+    return StorageClass::ONEZONE_IA;
+  else if (storage_class_str == "INTELLIGENT_TIERING")
+    return StorageClass::INTELLIGENT_TIERING;
+  else if (storage_class_str == "GLACIER")
+    return StorageClass::GLACIER;
+  else if (storage_class_str == "DEEP_ARCHIVE")
+    return StorageClass::DEEP_ARCHIVE;
+  else if (storage_class_str == "OUTPOSTS")
+    return StorageClass::OUTPOSTS;
+  else if (storage_class_str == "GLACIER_IR")
+    return StorageClass::GLACIER_IR;
+  else if (storage_class_str == "SNOW")
+    return StorageClass::SNOW;
+  else if (storage_class_str == "EXPRESS_ONEZONE")
+    return StorageClass::EXPRESS_ONEZONE;
+  else
+    return StorageClass::NOT_SET;
+}
+
 }  // namespace
 
 using namespace tiledb::common;
@@ -192,7 +241,6 @@ S3::S3(
     : s3_params_(S3Parameters(config))
     , stats_(parent_stats->create_child("S3"))
     , state_(State::UNINITIALIZED)
-    , credentials_provider_(nullptr)
     , file_buffer_size_(
           s3_params_.multipart_part_size_ * s3_params_.max_parallel_ops_)
     , vfs_thread_pool_(thread_pool)
@@ -200,6 +248,7 @@ S3::S3(
           s3_params_.requester_pays_ ? Aws::S3::Model::RequestPayer::requester :
                                        Aws::S3::Model::RequestPayer::NOT_SET)
     , sse_(Aws::S3::Model::ServerSideEncryption::NOT_SET)
+    , storage_class_(S3_StorageClass_from_str(s3_params_.storage_class_))
     , object_canned_acl_(
           S3_ObjectCannedACL_from_str(s3_params_.object_acl_str_))
     , bucket_canned_acl_(
@@ -511,6 +560,10 @@ void S3::touch(const URI& uri) const {
   if (!s3_params_.sse_kms_key_id_.empty())
     put_object_request.SetSSEKMSKeyId(
         Aws::String(s3_params_.sse_kms_key_id_.c_str()));
+  // TODO: These checks are not needed since AWS SDK 1.11.275
+  // https://github.com/aws/aws-sdk-cpp/pull/2875
+  if (storage_class_ != Aws::S3::Model::StorageClass::NOT_SET)
+    put_object_request.SetStorageClass(storage_class_);
   if (object_canned_acl_ != Aws::S3::Model::ObjectCannedACL::NOT_SET) {
     put_object_request.SetACL(object_canned_acl_);
   }
@@ -588,8 +641,7 @@ void S3::write(
   }
 }
 
-tuple<Status, optional<std::vector<directory_entry>>> S3::ls_with_sizes(
-    const URI& parent) const {
+std::vector<directory_entry> S3::ls_with_sizes(const URI& parent) const {
   return ls_with_sizes(parent, "/", -1);
 }
 
@@ -868,27 +920,22 @@ Status S3::ls(
     std::vector<std::string>* paths,
     const std::string& delimiter,
     int max_paths) const {
-  auto&& [st, entries] = ls_with_sizes(prefix, delimiter, max_paths);
-  RETURN_NOT_OK(st);
-
-  for (auto& fs : *entries) {
+  for (auto& fs : ls_with_sizes(prefix, delimiter, max_paths)) {
     paths->emplace_back(fs.path().native());
   }
 
   return Status::Ok();
 }
 
-tuple<Status, optional<std::vector<directory_entry>>> S3::ls_with_sizes(
+std::vector<directory_entry> S3::ls_with_sizes(
     const URI& prefix, const std::string& delimiter, int max_paths) const {
-  RETURN_NOT_OK_TUPLE(init_client(), nullopt);
+  throw_if_not_ok(init_client());
 
   const auto prefix_dir = prefix.add_trailing_slash();
 
   auto prefix_str = prefix_dir.to_string();
   if (!prefix_dir.is_s3()) {
-    auto st = LOG_STATUS(
-        Status_S3Error(std::string("URI is not an S3 URI: " + prefix_str)));
-    return {st, nullopt};
+    throw S3Exception("URI is not an S3 URI: " + prefix_str);
   }
 
   Aws::Http::URI aws_uri = prefix_str.c_str();
@@ -914,11 +961,10 @@ tuple<Status, optional<std::vector<directory_entry>>> S3::ls_with_sizes(
     auto list_objects_outcome = client_->ListObjectsV2(list_objects_request);
 
     if (!list_objects_outcome.IsSuccess()) {
-      auto st = LOG_STATUS(Status_S3Error(
+      throw S3Exception(
           std::string("Error while listing with prefix '") + prefix_str +
           "' and delimiter '" + delimiter + "'" +
-          outcome_error_message(list_objects_outcome)));
-      return {st, nullopt};
+          outcome_error_message(list_objects_outcome));
     }
 
     for (const auto& object : list_objects_outcome.GetResult().GetContents()) {
@@ -946,16 +992,15 @@ tuple<Status, optional<std::vector<directory_entry>>> S3::ls_with_sizes(
       Aws::String next_marker =
           list_objects_outcome.GetResult().GetNextContinuationToken();
       if (next_marker.empty()) {
-        auto st =
-            LOG_STATUS(Status_S3Error("Failed to retrieve next continuation "
-                                      "token for ListObjectsV2 request."));
-        return {st, nullopt};
+        throw S3Exception(
+            "Failed to retrieve next continuation "
+            "token for ListObjectsV2 request.");
       }
       list_objects_request.SetContinuationToken(std::move(next_marker));
     }
   }
 
-  return {Status::Ok(), entries};
+  return entries;
 }
 
 Status S3::move_object(const URI& old_uri, const URI& new_uri) const {
@@ -1274,6 +1319,10 @@ Status S3::init_client() const {
 
   std::lock_guard<std::mutex> lck(client_init_mtx_);
 
+  if (client_ != nullptr) {
+    return Status::Ok();
+  }
+
   auto s3_config_source = s3_params_.config_source_;
   if (s3_config_source != "auto" && s3_config_source != "config_files" &&
       s3_config_source != "sts_profile_with_web_identity") {
@@ -1283,28 +1332,12 @@ Status S3::init_client() const {
         "'sts_profile_with_web_identity'");
   }
 
-  if (client_ != nullptr) {
-    // Check credentials. If expired, refresh it
-    if (credentials_provider_) {
-      Aws::Auth::AWSCredentials credentials =
-          credentials_provider_->GetAWSCredentials();
-      if (credentials.IsExpired()) {
-        throw S3Exception("AWS credentials are expired.");
-      } else if (!s3_params_.no_sign_request_ && credentials.IsEmpty()) {
-        throw S3Exception(
-            "AWS credentials were not provided. For public S3 data, consider "
-            "setting the vfs.s3.no_sign_request config option.");
-      }
-    }
-    return Status::Ok();
-  }
-
   // ClientConfiguration should be lazily init'ed here in init_client to avoid
   // potential slowdowns for non s3 users as the ClientConfig now attempts to
   // check for client configuration on create, which can be slow if aws is not
   // configured on a users systems due to ec2 metadata check
 
-  client_config_ = make_shared<Aws::Client::ClientConfiguration>(HERE());
+  client_config_ = make_shared<Aws::S3::S3ClientConfiguration>(HERE());
 
   s3_tp_executor_ = make_shared<S3ThreadPoolExecutor>(HERE(), vfs_thread_pool_);
 
@@ -1344,11 +1377,35 @@ Status S3::init_client() const {
       s3_params_.connect_max_tries_,
       s3_params_.connect_scale_factor_);
 
+  client_config.useVirtualAddressing = s3_params_.use_virtual_addressing_;
+  client_config.payloadSigningPolicy =
+      Aws::Client::AWSAuthV4Signer::PayloadSigningPolicy::Never;
+
+  // Use a copy of the config with a different retry strategy for the
+  // credentials providers.
+  auto auth_config =
+      make_shared<Aws::STS::STSClientConfiguration>(HERE(), client_config);
+
+  auth_config->retryStrategy =
+      make_shared<Aws::Client::SpecifiedRetryableErrorsRetryStrategy>(
+          HERE(),
+          // Retry some errors that are retried by the providers' default retry
+          // strategies.
+          Aws::Vector<Aws::String>{
+              // STSAssumeRoleWebIdentityCredentialsProvider
+              "IDPCommunicationError",
+              "InvalidToken",
+              // SSOCredentialsProvider
+              "TooManyRequestsException"},
+          s3_params_.connect_max_tries_);
+
+  shared_ptr<Aws::Auth::AWSCredentialsProvider> credentials_provider;
+
   // If the user says not to sign a request, use the
   // AnonymousAWSCredentialsProvider This is equivalent to --no-sign-request on
   // the aws cli
   if (s3_params_.no_sign_request_) {
-    credentials_provider_ =
+    credentials_provider =
         make_shared<Aws::Auth::AnonymousAWSCredentialsProvider>(HERE());
   } else {  // Check other authentication methods
     switch ((!s3_params_.aws_access_key_id_.empty() ? 1 : 0) +
@@ -1357,6 +1414,9 @@ Status S3::init_client() const {
             (s3_config_source == "config_files" ? 8 : 0) +
             (s3_config_source == "sts_profile_with_web_identity" ? 16 : 0)) {
       case 0:
+        credentials_provider = make_shared<
+            tiledb::sm::filesystem::s3::DefaultAWSCredentialsProviderChain>(
+            HERE(), auth_config);
         break;
       case 1:
       case 2:
@@ -1371,7 +1431,7 @@ Status S3::init_client() const {
             !s3_params_.aws_session_token_.empty() ?
                 s3_params_.aws_session_token_.c_str() :
                 "");
-        credentials_provider_ =
+        credentials_provider =
             make_shared<Aws::Auth::SimpleAWSCredentialsProvider>(
                 HERE(), access_key_id, secret_access_key, session_token);
         break;
@@ -1392,14 +1452,14 @@ Status S3::init_client() const {
             !s3_params_.aws_session_name_.empty() ?
                 s3_params_.aws_session_name_.c_str() :
                 "");
-        credentials_provider_ =
+        credentials_provider =
             make_shared<Aws::Auth::STSAssumeRoleCredentialsProvider>(
                 HERE(),
                 role_arn,
                 session_name,
                 external_id,
                 load_frequency,
-                make_shared<Aws::STS::STSClient>(HERE(), client_config));
+                make_shared<Aws::STS::STSClient>(HERE(), *auth_config));
         break;
       }
       case 7: {
@@ -1410,20 +1470,24 @@ Status S3::init_client() const {
             "temporary authentication credentials are configured");
       }
       case 8: {
-        credentials_provider_ =
+        credentials_provider =
             make_shared<Aws::Auth::ProfileConfigFileAWSCredentialsProvider>(
                 HERE());
         break;
       }
       case 16: {
-        credentials_provider_ = make_shared<
+        credentials_provider = make_shared<
             Aws::Auth::STSProfileWithWebIdentityCredentialsProvider>(
             HERE(),
             Aws::Auth::GetConfigProfileName(),
             std::chrono::minutes(60),
-            [client_config](const auto& credentials) {
+            [config = auth_config](const auto& credentials) {
               return make_shared<Aws::STS::STSClient>(
-                  HERE(), credentials, client_config);
+                  HERE(),
+                  credentials,
+                  // Create default endpoint provider.
+                  make_shared<Aws::STS::STSEndpointProvider>(HERE()),
+                  *config);
             });
         break;
       }
@@ -1447,22 +1511,9 @@ Status S3::init_client() const {
   static std::mutex static_client_init_mtx;
   {
     std::lock_guard<std::mutex> static_lck(static_client_init_mtx);
-    if (credentials_provider_ == nullptr) {
-      client_ = make_shared<TileDBS3Client>(
-          HERE(),
-          s3_params_,
-          *client_config_,
-          Aws::Client::AWSAuthV4Signer::PayloadSigningPolicy::Never,
-          s3_params_.use_virtual_addressing_);
-    } else {
-      client_ = make_shared<TileDBS3Client>(
-          HERE(),
-          s3_params_,
-          credentials_provider_,
-          *client_config_,
-          Aws::Client::AWSAuthV4Signer::PayloadSigningPolicy::Never,
-          s3_params_.use_virtual_addressing_);
-    }
+    assert(credentials_provider);
+    client_ = make_shared<TileDBS3Client>(
+        HERE(), s3_params_, credentials_provider, client_config);
   }
 
   return Status::Ok();
@@ -1562,6 +1613,8 @@ Status S3::initiate_multipart_request(
   if (!s3_params_.sse_kms_key_id_.empty())
     multipart_upload_request.SetSSEKMSKeyId(
         Aws::String(s3_params_.sse_kms_key_id_.c_str()));
+  if (storage_class_ != Aws::S3::Model::StorageClass::NOT_SET)
+    multipart_upload_request.SetStorageClass(storage_class_);
   if (object_canned_acl_ != Aws::S3::Model::ObjectCannedACL::NOT_SET) {
     multipart_upload_request.SetACL(object_canned_acl_);
   }
@@ -1740,12 +1793,6 @@ void S3::write_direct(const URI& uri, const void* buffer, uint64_t length) {
   put_object_request.SetBody(stream);
   put_object_request.SetContentLength(length);
 
-  // we only want to hash once, and must do it after setting the body
-  auto md5_hash =
-      Aws::Utils::HashingUtils::CalculateMD5(*put_object_request.GetBody());
-
-  put_object_request.SetContentMD5(
-      Aws::Utils::HashingUtils::Base64Encode(md5_hash));
   put_object_request.SetContentType("application/octet-stream");
   put_object_request.SetBucket(aws_uri.GetAuthority());
   put_object_request.SetKey(aws_uri.GetPath());
@@ -1756,6 +1803,8 @@ void S3::write_direct(const URI& uri, const void* buffer, uint64_t length) {
   if (!s3_params_.sse_kms_key_id_.empty())
     put_object_request.SetSSEKMSKeyId(
         Aws::String(s3_params_.sse_kms_key_id_.c_str()));
+  if (storage_class_ != Aws::S3::Model::StorageClass::NOT_SET)
+    put_object_request.SetStorageClass(storage_class_);
   if (object_canned_acl_ != Aws::S3::Model::ObjectCannedACL::NOT_SET) {
     put_object_request.SetACL(object_canned_acl_);
   }
@@ -1767,16 +1816,6 @@ void S3::write_direct(const URI& uri, const void* buffer, uint64_t length) {
         outcome_error_message(put_object_outcome));
   }
 
-  // verify the MD5 hash of the result
-  // note the etag is hex-encoded not base64
-  Aws::StringStream md5_hex;
-  md5_hex << "\"" << Aws::Utils::HashingUtils::HexEncode(md5_hash) << "\"";
-  if (md5_hex.str() != put_object_outcome.GetResult().GetETag()) {
-    throw S3Exception(
-        "Object uploaded successfully, but MD5 hash does "
-        "not match result from server!' ");
-  }
-
   throw_if_not_ok(wait_for_object_to_propagate(
       put_object_request.GetBucket(), put_object_request.GetKey()));
 }
@@ -1926,8 +1965,6 @@ S3::MakeUploadPartCtx S3::make_upload_part_req(
   upload_part_request.SetPartNumber(upload_part_num);
   upload_part_request.SetUploadId(upload_id);
   upload_part_request.SetBody(stream);
-  upload_part_request.SetContentMD5(Aws::Utils::HashingUtils::Base64Encode(
-      Aws::Utils::HashingUtils::CalculateMD5(*stream)));
   upload_part_request.SetContentLength(length);
   if (request_payer_ != Aws::S3::Model::RequestPayer::NOT_SET)
     upload_part_request.SetRequestPayer(request_payer_);
