@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2022 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,6 +33,7 @@
 #include "tiledb/sm/query/legacy/reader.h"
 #include "tiledb/common/logger.h"
 #include "tiledb/sm/array/array.h"
+#include "tiledb/sm/array/array_operations.h"
 #include "tiledb/sm/array_schema/array_schema.h"
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/fragment/fragment_metadata.h"
@@ -40,7 +41,6 @@
 #include "tiledb/sm/misc/hilbert.h"
 #include "tiledb/sm/misc/parallel_functions.h"
 #include "tiledb/sm/misc/types.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/hilbert_order.h"
 #include "tiledb/sm/query/legacy/read_cell_slab_iter.h"
 #include "tiledb/sm/query/query_macros.h"
@@ -48,7 +48,6 @@
 #include "tiledb/sm/query/readers/result_tile.h"
 #include "tiledb/sm/query/update_value.h"
 #include "tiledb/sm/stats/global_stats.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
 #include "tiledb/sm/subarray/cell_slab.h"
 #include "tiledb/sm/tile/generic_tile_io.h"
 #include "tiledb/type/apply_with_type.h"
@@ -57,12 +56,11 @@ using namespace tiledb;
 using namespace tiledb::common;
 using namespace tiledb::sm::stats;
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
-class ReaderStatusException : public StatusException {
+class ReaderException : public StatusException {
  public:
-  explicit ReaderStatusException(const std::string& message)
+  explicit ReaderException(const std::string& message)
       : StatusException("Reader", message) {
   }
 };
@@ -111,23 +109,18 @@ Reader::Reader(
     bool remote_query)
     : ReaderBase(stats, logger->clone("Reader", ++logger_id_), params) {
   // Sanity checks
-  if (storage_manager_ == nullptr) {
-    throw ReaderStatusException(
-        "Cannot initialize reader; Storage manager not set");
-  }
-
   if (!params.default_channel_aggregates().empty()) {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize reader; Reader cannot process aggregates");
   }
 
   if (!params.skip_checks_serialization() && buffers_.empty()) {
-    throw ReaderStatusException("Cannot initialize reader; Buffers not set");
+    throw ReaderException("Cannot initialize reader; Buffers not set");
   }
 
   if (!params.skip_checks_serialization() && array_schema_.dense() &&
       !subarray_.is_set()) {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize reader; Dense reads must have a subarray set");
   }
 
@@ -143,6 +136,66 @@ Reader::Reader(
   check_validity_buffer_sizes();
 }
 
+/* ********************************* */
+/*          STATIC FUNCTIONS         */
+/* ********************************* */
+
+template <class T>
+void Reader::compute_result_space_tiles(
+    const std::vector<shared_ptr<FragmentMetadata>>& fragment_metadata,
+    const std::vector<std::vector<uint8_t>>& tile_coords,
+    const TileDomain<T>& array_tile_domain,
+    const std::vector<TileDomain<T>>& frag_tile_domains,
+    std::map<const T*, ResultSpaceTile<T>>& result_space_tiles,
+    shared_ptr<MemoryTracker> memory_tracker) {
+  auto fragment_num = (unsigned)frag_tile_domains.size();
+  auto dim_num = array_tile_domain.dim_num();
+  std::vector<T> start_coords;
+  const T* coords;
+  start_coords.resize(dim_num);
+
+  // For all tile coordinates
+  for (const auto& tc : tile_coords) {
+    coords = (T*)(&(tc[0]));
+    start_coords = array_tile_domain.start_coords(coords);
+
+    // Create result space tile and insert into the map
+    auto r =
+        result_space_tiles.emplace(coords, ResultSpaceTile<T>(memory_tracker));
+    auto& result_space_tile = r.first->second;
+    result_space_tile.set_start_coords(start_coords);
+
+    // Add fragment info to the result space tile
+    for (unsigned f = 0; f < fragment_num; ++f) {
+      // Check if the fragment overlaps with the space tile
+      if (!frag_tile_domains[f].in_tile_domain(coords))
+        continue;
+
+      // Check if any previous fragment covers this fragment
+      // for the tile identified by `coords`
+      bool covered = false;
+      for (unsigned j = 0; j < f; ++j) {
+        if (frag_tile_domains[j].covers(coords, frag_tile_domains[f])) {
+          covered = true;
+          break;
+        }
+      }
+
+      // Exclude this fragment from the space tile
+      if (covered)
+        continue;
+
+      // Include this fragment in the space tile
+      auto frag_domain = frag_tile_domains[f].domain_slice();
+      auto frag_idx = frag_tile_domains[f].id();
+      result_space_tile.append_frag_domain(frag_idx, frag_domain);
+      auto tile_idx = frag_tile_domains[f].tile_pos(coords);
+      result_space_tile.set_result_tile(
+          frag_idx, tile_idx, *fragment_metadata[frag_idx].get());
+    }
+  }
+}
+
 /* ****************************** */
 /*               API              */
 /* ****************************** */
@@ -277,10 +330,9 @@ Status Reader::load_initial_data() {
   }
 
   // Load delete conditions.
-  auto&& [st, conditions, update_values] =
-      storage_manager_->load_delete_and_update_conditions(*array_);
-  RETURN_CANCEL_OR_ERROR(st);
-  delete_and_update_conditions_ = std::move(*conditions);
+  auto&& [conditions, update_values] =
+      load_delete_and_update_conditions(resources_, *array_.get());
+  delete_and_update_conditions_ = conditions;
 
   // Set timestamps variables
   user_requested_timestamps_ = buffers_.count(constants::timestamps) != 0 ||
@@ -519,10 +571,10 @@ Status Reader::compute_range_result_coords(
     }
   }
 
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, range_num, [&](uint64_t r) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, range_num, [&](uint64_t r) {
         // Compute overlapping coordinates per range
-        RETURN_NOT_OK(compute_range_result_coords(
+        throw_if_not_ok(compute_range_result_coords(
             subarray,
             r,
             result_tile_map,
@@ -532,12 +584,14 @@ Status Reader::compute_range_result_coords(
         // Dedup unless there is a single fragment or array schema allows
         // duplicates
         if (!single_fragment[r] && !allows_dups) {
-          RETURN_CANCEL_OR_ERROR(sort_result_coords(
+          throw_if_not_ok(sort_result_coords(
               range_result_coords[r].begin(),
               range_result_coords[r].end(),
               range_result_coords[r].size(),
               sort_layout));
-          RETURN_CANCEL_OR_ERROR(dedup_result_coords(range_result_coords[r]));
+          throw_if_cancelled();
+          throw_if_not_ok(dedup_result_coords(range_result_coords[r]));
+          throw_if_cancelled();
         }
 
         return Status::Ok();
@@ -617,15 +671,16 @@ Status Reader::compute_range_result_coords(
   // Gather result range coordinates per fragment
   auto fragment_num = fragment_metadata_.size();
   std::vector<std::vector<ResultCoords>> range_result_coords_vec(fragment_num);
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, fragment_num, [&](uint32_t f) {
-        return compute_range_result_coords(
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, fragment_num, [&](uint32_t f) {
+        throw_if_not_ok(compute_range_result_coords(
             subarray,
             range_idx,
             f,
             result_tile_map,
             result_tiles,
-            range_result_coords_vec[f]);
+            range_result_coords_vec[f]));
+        return Status::Ok();
       });
   RETURN_NOT_OK(status);
 
@@ -914,7 +969,7 @@ Status Reader::copy_fixed_cells(
       &cs_offsets,
       fixed_cs_partitions);
   auto status = parallel_for(
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       0,
       fixed_cs_partitions->size(),
       std::move(copy_fn));
@@ -938,10 +993,8 @@ void Reader::compute_fixed_cs_partitions(
     return;
   }
 
-  const auto num_copy_threads =
-      storage_manager_->compute_tp()->concurrency_level();
-
   // Calculate the partition sizes.
+  const auto num_copy_threads = resources_.compute_tp().concurrency_level();
   auto num_cs = result_cell_slabs.size();
   const uint64_t num_cs_partitions =
       std::min<uint64_t>(num_copy_threads, num_cs);
@@ -1106,7 +1159,7 @@ Status Reader::copy_var_cells(
       &var_offsets_per_cs,
       var_cs_partitions);
   auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, var_cs_partitions->size(), copy_fn);
+      &resources_.compute_tp(), 0, var_cs_partitions->size(), copy_fn);
 
   RETURN_NOT_OK(status);
 
@@ -1127,10 +1180,8 @@ void Reader::compute_var_cs_partitions(
     return;
   }
 
-  const auto num_copy_threads =
-      storage_manager_->compute_tp()->concurrency_level();
-
   // Calculate the partition range.
+  const auto num_copy_threads = resources_.compute_tp().concurrency_level();
   const uint64_t num_cs = result_cell_slabs.size();
   const uint64_t num_cs_partitions =
       std::min<uint64_t>(num_copy_threads, num_cs);
@@ -1480,6 +1531,23 @@ Status Reader::process_tiles(
   return Status::Ok();
 }
 
+template <class T>
+void Reader::compute_result_space_tiles(
+    const Subarray& subarray,
+    const Subarray& partitioner_subarray,
+    std::map<const T*, ResultSpaceTile<T>>& result_space_tiles) const {
+  auto&& [array_tile_domain, frag_tile_domains] =
+      compute_tile_domains<T>(partitioner_subarray);
+
+  compute_result_space_tiles<T>(
+      fragment_metadata_,
+      subarray.tile_coords(),
+      array_tile_domain,
+      frag_tile_domains,
+      result_space_tiles,
+      query_memory_tracker_);
+}
+
 template <class T>
 Status Reader::compute_result_cell_slabs(
     const Subarray& subarray,
@@ -1827,7 +1895,7 @@ void Reader::init_read_state() {
   // Check subarray
   if (subarray_.layout() == Layout::GLOBAL_ORDER &&
       subarray_.range_num() != 1) {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize read state; Multi-range subarrays do not support "
         "global order");
   }
@@ -1836,21 +1904,21 @@ void Reader::init_read_state() {
   bool found = false;
   uint64_t memory_budget = 0;
   if (!config_.get<uint64_t>("sm.memory_budget", &memory_budget, &found).ok()) {
-    throw ReaderStatusException("Cannot get setting");
+    throw ReaderException("Cannot get setting");
   }
   assert(found);
 
   uint64_t memory_budget_var = 0;
   if (!config_.get<uint64_t>("sm.memory_budget_var", &memory_budget_var, &found)
            .ok()) {
-    throw ReaderStatusException("Cannot get setting");
+    throw ReaderException("Cannot get setting");
   }
   assert(found);
 
   offsets_format_mode_ = config_.get("sm.var_offsets.mode", &found);
   assert(found);
   if (offsets_format_mode_ != "bytes" && offsets_format_mode_ != "elements") {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize reader; Unsupported offsets"
         " format in configuration");
   }
@@ -1859,19 +1927,19 @@ void Reader::init_read_state() {
            .get<bool>(
                "sm.var_offsets.extra_element", &offsets_extra_element_, &found)
            .ok()) {
-    throw ReaderStatusException("Cannot get setting");
+    throw ReaderException("Cannot get setting");
   }
   assert(found);
 
   if (!config_
            .get<uint32_t>("sm.var_offsets.bitsize", &offsets_bitsize_, &found)
            .ok()) {
-    throw ReaderStatusException("Cannot get setting");
+    throw ReaderException("Cannot get setting");
   }
   assert(found);
 
   if (offsets_bitsize_ != 32 && offsets_bitsize_ != 64) {
-    throw ReaderStatusException(
+    throw ReaderException(
         "Cannot initialize reader; Unsupported offsets"
         " bitsize in configuration");
   }
@@ -1889,7 +1957,7 @@ void Reader::init_read_state() {
       memory_budget,
       memory_budget_var,
       memory_budget_validity,
-      storage_manager_->compute_tp(),
+      &resources_.compute_tp(),
       stats_,
       logger_);
   read_state_.overflowed_ = false;
@@ -1906,14 +1974,14 @@ void Reader::init_read_state() {
         if (!read_state_.partitioner_
                  .set_result_budget(attr_name.c_str(), *buffer_size)
                  .ok()) {
-          throw ReaderStatusException("Cannot set result budget");
+          throw ReaderException("Cannot set result budget");
         }
       } else {
         if (!read_state_.partitioner_
                  .set_result_budget_nullable(
                      attr_name.c_str(), *buffer_size, *buffer_validity_size)
                  .ok()) {
-          throw ReaderStatusException("Cannot set result budget");
+          throw ReaderException("Cannot set result budget");
         }
       }
     } else {
@@ -1922,7 +1990,7 @@ void Reader::init_read_state() {
                  .set_result_budget(
                      attr_name.c_str(), *buffer_size, *buffer_var_size)
                  .ok()) {
-          throw ReaderStatusException("Cannot set result budget");
+          throw ReaderException("Cannot set result budget");
         }
       } else {
         if (!read_state_.partitioner_
@@ -1932,7 +2000,7 @@ void Reader::init_read_state() {
                      *buffer_var_size,
                      *buffer_validity_size)
                  .ok()) {
-          throw ReaderStatusException("Cannot set result budget");
+          throw ReaderException("Cannot set result budget");
         }
       }
     }
@@ -1953,26 +2021,23 @@ Status Reader::sort_result_coords(
 
   if (layout == Layout::ROW_MAJOR) {
     parallel_sort(
-        storage_manager_->compute_tp(), iter_begin, iter_end, RowCmp(domain));
+        &resources_.compute_tp(), iter_begin, iter_end, RowCmp(domain));
   } else if (layout == Layout::COL_MAJOR) {
     parallel_sort(
-        storage_manager_->compute_tp(), iter_begin, iter_end, ColCmp(domain));
+        &resources_.compute_tp(), iter_begin, iter_end, ColCmp(domain));
   } else if (layout == Layout::GLOBAL_ORDER) {
     if (array_schema_.cell_order() == Layout::HILBERT) {
       std::vector<std::pair<uint64_t, uint64_t>> hilbert_values(coords_num);
       RETURN_NOT_OK(calculate_hilbert_values(iter_begin, &hilbert_values));
       parallel_sort(
-          storage_manager_->compute_tp(),
+          &resources_.compute_tp(),
           hilbert_values.begin(),
           hilbert_values.end(),
           HilbertCmpRCI(domain, iter_begin));
       RETURN_NOT_OK(reorganize_result_coords(iter_begin, &hilbert_values));
     } else {
       parallel_sort(
-          storage_manager_->compute_tp(),
-          iter_begin,
-          iter_end,
-          GlobalCmp(domain));
+          &resources_.compute_tp(), iter_begin, iter_end, GlobalCmp(domain));
     }
   } else {
     assert(false);
@@ -2113,8 +2178,8 @@ Status Reader::calculate_hilbert_values(
   auto coords_num = (uint64_t)hilbert_values->size();
 
   // Calculate Hilbert values in parallel
-  auto status = parallel_for(
-      storage_manager_->compute_tp(), 0, coords_num, [&](uint64_t c) {
+  auto status =
+      parallel_for(&resources_.compute_tp(), 0, coords_num, [&](uint64_t c) {
         std::vector<uint64_t> coords(dim_num);
         for (uint32_t d = 0; d < dim_num; ++d) {
           auto dim{array_schema_.dimension_ptr(d)};
@@ -2399,5 +2464,4 @@ void Reader::fill_dense_coords_col_slab(
   }
 }
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
