@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2021 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -40,10 +40,8 @@
 #include "tiledb/sm/array_schema/dimension.h"
 #include "tiledb/sm/query/query_condition.h"
 #include "tiledb/sm/query/readers/result_cell_slab.h"
-#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 
-namespace tiledb {
-namespace sm {
+namespace tiledb::sm {
 
 class Array;
 class ArraySchema;
@@ -110,7 +108,12 @@ class MemoryBudget {
 
   MemoryBudget() = delete;
 
-  MemoryBudget(Config& config, std::string reader_string) {
+  MemoryBudget(
+      Config& config,
+      std::string reader_string,
+      optional<uint64_t> total_budget)
+      : total_budget_(total_budget.value_or(0))
+      , memory_budget_from_query_(total_budget) {
     refresh_config(config, reader_string);
   }
 
@@ -128,8 +131,10 @@ class MemoryBudget {
    * @param reader_string String to identify the reader settings to load.
    */
   void refresh_config(Config& config, std::string reader_string) {
-    total_budget_ =
-        config.get<uint64_t>("sm.mem.total_budget", Config::must_find);
+    if (!memory_budget_from_query_.has_value()) {
+      total_budget_ =
+          config.get<uint64_t>("sm.mem.total_budget", Config::must_find);
+    }
 
     ratio_coords_ = config.get<double>(
         "sm.mem.reader." + reader_string + ".ratio_coords", Config::must_find);
@@ -151,6 +156,14 @@ class MemoryBudget {
     return total_budget_;
   }
 
+  /**
+   * @return Portion of the total memory budget dedicated to loading coordinate
+   * tiles.
+   */
+  double coordinates_budget() {
+    return total_budget_ * ratio_coords_;
+  }
+
   /**
    * @return Ratio of the budget dedicated to loading coordinate tiles into
    * memory.
@@ -192,6 +205,9 @@ class MemoryBudget {
   /** Total memory budget. */
   uint64_t total_budget_;
 
+  /** Total memory budget if overridden by the query. */
+  optional<uint64_t> memory_budget_from_query_;
+
   /** How much of the memory budget is reserved for coords. */
   double ratio_coords_;
 
@@ -296,11 +312,79 @@ class SparseIndexReaderBase : public ReaderBase {
    * it is really required to determine if a query is incomplete from the client
    * side of a cloud request.
    */
-  struct ReadState {
+  class ReadState {
+   public:
+    /* ********************************* */
+    /*     CONSTRUCTORS & DESTRUCTORS    */
+    /* ********************************* */
+
+    /** Delete default constructor. */
+    ReadState() = delete;
+
+    /** Constructor.
+     * @param frag_idxs_len The length of the fragment index vector.
+     */
+    ReadState(size_t frag_idxs_len)
+        : frag_idx_(frag_idxs_len) {
+    }
+
+    /** Constructor used in deserialization. */
+    ReadState(std::vector<FragIdx>&& frag_idx, bool done_adding_result_tiles)
+        : frag_idx_(std::move(frag_idx))
+        , done_adding_result_tiles_(done_adding_result_tiles) {
+    }
+
+    /* ********************************* */
+    /*                API                */
+    /* ********************************* */
+
+    /**
+     * Return whether the tiles that will be processed are loaded in memory.
+     * @return Done adding result tiles.
+     */
+    inline bool done_adding_result_tiles() const {
+      return done_adding_result_tiles_;
+    }
+
+    /**
+     * Sets the flag that determines whether the tiles that will be processed
+     * are loaded in memory.
+     * @param done_adding_result_tiles Done adding result tiles.
+     */
+    inline void set_done_adding_result_tiles(bool done_adding_result_tiles) {
+      done_adding_result_tiles_ = done_adding_result_tiles;
+    }
+
+    /**
+     * Sets a value in the fragment index vector.
+     * @param idx The index of the vector.
+     * @param val The value to set frag_idx[idx] to.
+     */
+    inline void set_frag_idx(uint64_t idx, FragIdx val) {
+      if (idx >= frag_idx_.size()) {
+        throw std::runtime_error(
+            "ReadState::set_frag_idx: idx greater than frag_idx_'s size.");
+      }
+      frag_idx_[idx] = std::move(val);
+    }
+
+    /**
+     * Returns a read-only version of the fragment index vector.
+     * @return The fragment index vector.
+     */
+    const std::vector<FragIdx>& frag_idx() const {
+      return frag_idx_;
+    }
+
+    /* ********************************* */
+    /*         PRIVATE ATTRIBUTES        */
+    /* ********************************* */
+
+   private:
     /** The tile index inside of each fragments. */
     std::vector<FragIdx> frag_idx_;
 
-    /** Is the reader done with the query. */
+    /** Have all tiles to be processed been loaded in memory? */
     bool done_adding_result_tiles_;
   };
 
@@ -506,16 +590,16 @@ class SparseIndexReaderBase : public ReaderBase {
   /**
    * Returns the current read state.
    *
-   * @return pointer to the read state.
+   * @return const reference to the read state.
    */
-  const ReadState* read_state() const;
+  const ReadState& read_state() const;
 
   /**
-   * Returns the current read state.
+   * Sets the new read state. Used only for deserialization.
    *
-   * @return pointer to the read state.
+   * @param read_state New read_state value.
    */
-  ReadState* read_state();
+  void set_read_state(ReadState read_state);
 
  protected:
   /* ********************************* */
@@ -537,24 +621,12 @@ class SparseIndexReaderBase : public ReaderBase {
   /** Dimension names. */
   std::vector<std::string> dim_names_;
 
-  /** Are dimensions var sized. */
-  std::vector<bool> is_dim_var_size_;
-
-  /** Memory tracker object for the array. */
-  MemoryTracker* array_memory_tracker_;
-
   /** Memory used for coordinates tiles. */
   std::atomic<uint64_t> memory_used_for_coords_total_;
 
   /** Are we in elements mode. */
   bool elements_mode_;
 
-  /** Names of dim/attr loaded for query condition. */
-  std::vector<std::string> qc_loaded_attr_names_;
-
-  /** Are we doing deletes consolidation (without purge option). */
-  bool deletes_consolidation_no_purge_;
-
   /** Do we allow partial tile offset loading for this query? */
   bool partial_tile_offsets_loading_;
 
@@ -564,9 +636,6 @@ class SparseIndexReaderBase : public ReaderBase {
   /** Attributes for which to load tile offsets. */
   std::vector<std::string> attr_tile_offsets_to_load_;
 
-  /** Per fragment tile offsets memory usage. */
-  std::vector<uint64_t> per_frag_tile_offsets_usage_;
-
   /* ********************************* */
   /*         PROTECTED METHODS         */
   /* ********************************* */
@@ -574,13 +643,6 @@ class SparseIndexReaderBase : public ReaderBase {
   /** @return Available memory. */
   uint64_t available_memory();
 
-  /**
-   * Computes the required size for loading tile offsets, per fragments.
-   *
-   * @return Required memory for loading tile offsets, per fragments.
-   */
-  std::vector<uint64_t> tile_offset_sizes();
-
   /**
    * Returns if there is any condition to be applied post deduplication. This
    * will return true if we have:
@@ -713,7 +775,6 @@ class SparseIndexReaderBase : public ReaderBase {
   void add_extra_offset();
 };
 
-}  // namespace sm
-}  // namespace tiledb
+}  // namespace tiledb::sm
 
 #endif  // TILEDB_SPARSE_INDEX_READER_BASE_H
