@@ -40,6 +40,7 @@
 // clang-format on
 
 #include "tiledb/sm/array_schema/enumeration.h"
+#include "tiledb/sm/array/array.h"
 #include "tiledb/sm/config/config.h"
 #include "tiledb/sm/enums/serialization_type.h"
 #include "tiledb/sm/serialization/enumeration.h"
@@ -48,6 +49,22 @@ using namespace tiledb::common;
 
 namespace tiledb::sm::serialization {
 
+class EnumerationSerializationException : public StatusException {
+ public:
+  explicit EnumerationSerializationException(const std::string& message)
+      : StatusException("[TileDB::Serialization][Enumeration]", message) {
+  }
+};
+
+class EnumerationSerializationDisabledException
+    : public EnumerationSerializationException {
+ public:
+  explicit EnumerationSerializationDisabledException()
+      : EnumerationSerializationException(
+            "Cannot (de)serialize; serialization not enabled.") {
+  }
+};
+
 #ifdef TILEDB_SERIALIZATION
 
 void enumeration_to_capnp(
@@ -138,28 +155,69 @@ std::vector<std::string> load_enumerations_request_from_capnp(
 
 void load_enumerations_response_to_capnp(
     capnp::LoadEnumerationsResponse::Builder& builder,
-    const std::vector<shared_ptr<const Enumeration>>& enumerations) {
-  auto num_enmrs = enumerations.size();
-  if (num_enmrs > 0) {
-    auto enmr_builders = builder.initEnumerations(num_enmrs);
-    for (size_t i = 0; i < num_enmrs; i++) {
+    const std::unordered_map<
+        std::string,
+        std::vector<shared_ptr<const Enumeration>>>& enumerations) {
+  auto num_schemas = enumerations.size();
+  // If there is only one schema, it is always the latest so we don't need to
+  // serialize the extra data.
+  if (num_schemas == 1) {
+    auto num_enmr = enumerations.begin()->second.size();
+    auto enmr_builders = builder.initEnumerations(num_enmr);
+    for (size_t i = 0; i < num_enmr; i++) {
       auto enmr_builder = enmr_builders[i];
-      enumeration_to_capnp(enumerations[i], enmr_builder);
+      enumeration_to_capnp(enumerations.begin()->second[i], enmr_builder);
+    }
+  } else if (num_schemas > 1) {
+    // If there were enumerations loaded on multiple schemas, serialize the full
+    // map of schema_names and their enumerations.
+    auto enmr_map_builder = builder.initAllEnumerations();
+    auto map_entry_builder = enmr_map_builder.initEntries(num_schemas);
+    for (size_t i = 0; const auto& entry : enumerations) {
+      auto num_enmr = entry.second.size();
+      // Set the map key to the schema name
+      map_entry_builder[i].setKey(entry.first);
+
+      // Build the list of enumerations that map to this schema name.
+      auto enmr_builders = map_entry_builder[i++].initValue(num_enmr);
+      for (size_t j = 0; j < num_enmr; j++) {
+        auto enmr_builder = enmr_builders[j];
+        enumeration_to_capnp(entry.second[j], enmr_builder);
+      }
     }
   }
 }
 
-std::vector<shared_ptr<const Enumeration>>
+std::unordered_map<std::string, std::vector<shared_ptr<const Enumeration>>>
 load_enumerations_response_from_capnp(
     const capnp::LoadEnumerationsResponse::Reader& reader,
+    const ArraySchema& array_schema,
     shared_ptr<MemoryTracker> memory_tracker) {
-  std::vector<shared_ptr<const Enumeration>> ret;
+  std::unordered_map<std::string, std::vector<shared_ptr<const Enumeration>>>
+      ret;
   if (reader.hasEnumerations()) {
+    std::vector<shared_ptr<const Enumeration>> loaded_enmrs;
     auto enmr_readers = reader.getEnumerations();
     for (auto enmr_reader : enmr_readers) {
-      ret.push_back(enumeration_from_capnp(enmr_reader, memory_tracker));
+      loaded_enmrs.push_back(
+          enumeration_from_capnp(enmr_reader, memory_tracker));
+    }
+    // The name of the latest array schema will not be serialized in the
+    // response if we are only loading enumerations from the latest schema.
+    return {{array_schema.name(), loaded_enmrs}};
+  } else if (reader.hasAllEnumerations()) {
+    auto all_enmrs_reader = reader.getAllEnumerations();
+    for (auto enmr_entry_reader : all_enmrs_reader.getEntries()) {
+      std::vector<shared_ptr<const Enumeration>> loaded_enmrs;
+      for (auto enmr_reader : enmr_entry_reader.getValue()) {
+        loaded_enmrs.push_back(
+            enumeration_from_capnp(enmr_reader, memory_tracker));
+      }
+
+      ret[enmr_entry_reader.getKey()] = loaded_enmrs;
     }
   }
+
   return ret;
 }
 
@@ -167,56 +225,45 @@ void serialize_load_enumerations_request(
     const Config& config,
     const std::vector<std::string>& enumeration_names,
     SerializationType serialize_type,
-    Buffer& request) {
+    SerializationBuffer& request) {
   try {
     ::capnp::MallocMessageBuilder message;
     capnp::LoadEnumerationsRequest::Builder builder =
         message.initRoot<capnp::LoadEnumerationsRequest>();
     load_enumerations_request_to_capnp(builder, config, enumeration_names);
 
-    request.reset_size();
-    request.reset_offset();
-
     switch (serialize_type) {
       case SerializationType::JSON: {
         ::capnp::JsonCodec json;
         kj::String capnp_json = json.encode(builder);
-        const auto json_len = capnp_json.size();
-        const char nul = '\0';
-        // size does not include needed null terminator, so add +1
-        throw_if_not_ok(request.realloc(json_len + 1));
-        throw_if_not_ok(request.write(capnp_json.cStr(), json_len));
-        throw_if_not_ok(request.write(&nul, 1));
+        request.assign_null_terminated(capnp_json);
         break;
       }
       case SerializationType::CAPNP: {
         kj::Array<::capnp::word> protomessage = messageToFlatArray(message);
-        kj::ArrayPtr<const char> message_chars = protomessage.asChars();
-        const auto nbytes = message_chars.size();
-        throw_if_not_ok(request.realloc(nbytes));
-        throw_if_not_ok(request.write(message_chars.begin(), nbytes));
+        request.assign(protomessage.asChars());
         break;
       }
       default: {
-        throw Status_SerializationError(
+        throw EnumerationSerializationException(
             "Error serializing load enumerations request; "
             "Unknown serialization type passed");
       }
     }
 
   } catch (kj::Exception& e) {
-    throw Status_SerializationError(
+    throw EnumerationSerializationException(
         "Error serializing load enumerations request; kj::Exception: " +
         std::string(e.getDescription().cStr()));
   } catch (std::exception& e) {
-    throw Status_SerializationError(
+    throw EnumerationSerializationException(
         "Error serializing load enumerations request; exception " +
         std::string(e.what()));
   }
 }
 
 std::vector<std::string> deserialize_load_enumerations_request(
-    SerializationType serialize_type, const Buffer& request) {
+    SerializationType serialize_type, span<const char> request) {
   try {
     switch (serialize_type) {
       case SerializationType::JSON: {
@@ -224,8 +271,7 @@ std::vector<std::string> deserialize_load_enumerations_request(
         ::capnp::MallocMessageBuilder message_builder;
         capnp::LoadEnumerationsRequest::Builder builder =
             message_builder.initRoot<capnp::LoadEnumerationsRequest>();
-        json.decode(
-            kj::StringPtr(static_cast<const char*>(request.data())), builder);
+        json.decode(kj::StringPtr(request.data()), builder);
         capnp::LoadEnumerationsRequest::Reader reader = builder.asReader();
         return load_enumerations_request_from_capnp(reader);
       }
@@ -239,77 +285,70 @@ std::vector<std::string> deserialize_load_enumerations_request(
         return load_enumerations_request_from_capnp(reader);
       }
       default: {
-        throw Status_SerializationError(
+        throw EnumerationSerializationException(
             "Error deserializing load enumerations request; "
             "Unknown serialization type passed");
       }
     }
   } catch (kj::Exception& e) {
-    throw Status_SerializationError(
+    throw EnumerationSerializationException(
         "Error deserializing load enumerations request; kj::Exception: " +
         std::string(e.getDescription().cStr()));
   } catch (std::exception& e) {
-    throw Status_SerializationError(
+    throw EnumerationSerializationException(
         "Error deserializing load enumerations request; exception " +
         std::string(e.what()));
   }
 }
 
 void serialize_load_enumerations_response(
-    const std::vector<shared_ptr<const Enumeration>>& enumerations,
+    const std::unordered_map<
+        std::string,
+        std::vector<shared_ptr<const Enumeration>>>& enumerations,
     SerializationType serialize_type,
-    Buffer& response) {
+    SerializationBuffer& response) {
   try {
     ::capnp::MallocMessageBuilder message;
     capnp::LoadEnumerationsResponse::Builder builder =
         message.initRoot<capnp::LoadEnumerationsResponse>();
     load_enumerations_response_to_capnp(builder, enumerations);
 
-    response.reset_size();
-    response.reset_offset();
-
     switch (serialize_type) {
       case SerializationType::JSON: {
         ::capnp::JsonCodec json;
         kj::String capnp_json = json.encode(builder);
-        const auto json_len = capnp_json.size();
-        const char nul = '\0';
-        // size does not include needed null terminator, so add +1
-        throw_if_not_ok(response.realloc(json_len + 1));
-        throw_if_not_ok(response.write(capnp_json.cStr(), json_len));
-        throw_if_not_ok(response.write(&nul, 1));
+        response.assign_null_terminated(capnp_json);
         break;
       }
       case SerializationType::CAPNP: {
         kj::Array<::capnp::word> protomessage = messageToFlatArray(message);
-        kj::ArrayPtr<const char> message_chars = protomessage.asChars();
-        const auto nbytes = message_chars.size();
-        throw_if_not_ok(response.realloc(nbytes));
-        throw_if_not_ok(response.write(message_chars.begin(), nbytes));
+        response.assign(protomessage.asChars());
         break;
       }
       default: {
-        throw Status_SerializationError(
+        throw EnumerationSerializationException(
             "Error serializing load enumerations response; "
             "Unknown serialization type passed");
       }
     }
 
   } catch (kj::Exception& e) {
-    throw Status_SerializationError(
+    throw EnumerationSerializationException(
         "Error serializing load enumerations response; kj::Exception: " +
         std::string(e.getDescription().cStr()));
   } catch (std::exception& e) {
-    throw Status_SerializationError(
+    throw EnumerationSerializationException(
         "Error serializing load enumerations response; exception " +
         std::string(e.what()));
   }
 }
 
-std::vector<shared_ptr<const Enumeration>>
+std::unordered_map<std::string, std::vector<shared_ptr<const Enumeration>>>
 deserialize_load_enumerations_response(
+    const ArraySchema& array_schema,
+    const Config& config,
     SerializationType serialize_type,
-    const Buffer& response,
+    span<const char> response,
     shared_ptr<MemoryTracker> memory_tracker) {
   try {
     switch (serialize_type) {
@@ -318,32 +357,42 @@ deserialize_load_enumerations_response(
         ::capnp::MallocMessageBuilder message_builder;
         capnp::LoadEnumerationsResponse::Builder builder =
             message_builder.initRoot<capnp::LoadEnumerationsResponse>();
-        json.decode(
-            kj::StringPtr(static_cast<const char*>(response.data())), builder);
+        json.decode(kj::StringPtr(response.data(), response.size()), builder);
         capnp::LoadEnumerationsResponse::Reader reader = builder.asReader();
-        return load_enumerations_response_from_capnp(reader, memory_tracker);
+        return load_enumerations_response_from_capnp(
+            reader, array_schema, memory_tracker);
       }
       case SerializationType::CAPNP: {
+        // Set traversal limit from config
+        uint64_t limit =
+            config.get<uint64_t>("rest.capnp_traversal_limit").value();
+        ::capnp::ReaderOptions readerOptions;
+        // capnp uses the limit in words
+        readerOptions.traversalLimitInWords = limit / sizeof(::capnp::word);
+
         const auto mBytes = reinterpret_cast<const kj::byte*>(response.data());
-        ::capnp::FlatArrayMessageReader array_reader(kj::arrayPtr(
-            reinterpret_cast<const ::capnp::word*>(mBytes),
-            response.size() / sizeof(::capnp::word)));
+        ::capnp::FlatArrayMessageReader array_reader(
+            kj::arrayPtr(
+                reinterpret_cast<const ::capnp::word*>(mBytes),
+                response.size() / sizeof(::capnp::word)),
+            readerOptions);
         capnp::LoadEnumerationsResponse::Reader reader =
             array_reader.getRoot<capnp::LoadEnumerationsResponse>();
-        return load_enumerations_response_from_capnp(reader, memory_tracker);
+        return load_enumerations_response_from_capnp(
+            reader, array_schema, memory_tracker);
       }
       default: {
-        throw Status_SerializationError(
+        throw EnumerationSerializationException(
             "Error deserializing load enumerations response; "
             "Unknown serialization type passed");
       }
     }
   } catch (kj::Exception& e) {
-    throw Status_SerializationError(
+    throw EnumerationSerializationException(
         "Error deserializing load enumerations response; kj::Exception: " +
         std::string(e.getDescription().cStr()));
   } catch (std::exception& e) {
-    throw Status_SerializationError(
+    throw EnumerationSerializationException(
         "Error deserializing load enumerations response; exception " +
         std::string(e.what()));
   }
@@ -355,30 +404,31 @@ void serialize_load_enumerations_request(
     const Config&,
     const std::vector<std::string>&,
     SerializationType,
-    Buffer&) {
-  throw Status_SerializationError(
-      "Cannot serialize; serialization not enabled.");
+    SerializationBuffer&) {
+  throw EnumerationSerializationDisabledException();
 }
 
 std::vector<std::string> deserialize_load_enumerations_request(
-    SerializationType, const Buffer&) {
-  throw Status_SerializationError(
-      "Cannot serialize; serialization not enabled.");
+    SerializationType, span<const char>) {
+  throw EnumerationSerializationDisabledException();
 }
 
 void serialize_load_enumerations_response(
-    const std::vector<shared_ptr<const Enumeration>>&,
+    const std::
+        unordered_map<std::string, std::vector<shared_ptr<const Enumeration>>>&,
     SerializationType,
-    Buffer&) {
-  throw Status_SerializationError(
-      "Cannot serialize; serialization not enabled.");
+    SerializationBuffer&) {
+  throw EnumerationSerializationDisabledException();
 }
 
-std::vector<shared_ptr<const Enumeration>>
+std::unordered_map<std::string, std::vector<shared_ptr<const Enumeration>>>
 deserialize_load_enumerations_response(
-    SerializationType, const Buffer&, shared_ptr<MemoryTracker>) {
-  throw Status_SerializationError(
-      "Cannot serialize; serialization not enabled.");
+    const Array&,
+    const Config&,
+    SerializationType,
+    span<const char>,
+    shared_ptr<MemoryTracker>) {
+  throw EnumerationSerializationDisabledException();
 }
 
 #endif  // TILEDB_SERIALIZATION
