@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -49,15 +49,15 @@
 #include "tiledb/sm/fragment/fragment_metadata.h"
 #include "tiledb/sm/misc/hash.h"
 #include "tiledb/sm/misc/parallel_functions.h"
+#include "tiledb/sm/misc/rectangle.h"
 #include "tiledb/sm/misc/resource_pool.h"
 #include "tiledb/sm/misc/tdb_math.h"
-#include "tiledb/sm/misc/utils.h"
 #include "tiledb/sm/query/query.h"
 #include "tiledb/sm/query/readers/sparse_index_reader_base.h"
 #include "tiledb/sm/rest/rest_client.h"
 #include "tiledb/sm/rtree/rtree.h"
 #include "tiledb/sm/stats/global_stats.h"
-#include "tiledb/sm/storage_manager/storage_manager.h"
+#include "tiledb/sm/storage_manager/context_resources.h"
 #include "tiledb/sm/subarray/relevant_fragment_generator.h"
 #include "tiledb/sm/subarray/subarray.h"
 #include "tiledb/type/apply_with_type.h"
@@ -88,22 +88,21 @@ Subarray::Subarray()
     , est_result_size_computed_(false)
     , relevant_fragments_(0)
     , coalesce_ranges_(true)
-    , ranges_sorted_(false) {
+    , ranges_sorted_(false)
+    , merge_overlapping_ranges_(true) {
 }
 
 Subarray::Subarray(
     const Array* array,
     Stats* const parent_stats,
     shared_ptr<Logger> logger,
-    const bool coalesce_ranges,
-    StorageManager* storage_manager)
+    const bool coalesce_ranges)
     : Subarray(
           array->opened_array(),
           Layout::UNORDERED,
           parent_stats,
           logger,
-          coalesce_ranges,
-          storage_manager) {
+          coalesce_ranges) {
 }
 
 Subarray::Subarray(
@@ -111,15 +110,13 @@ Subarray::Subarray(
     const Layout layout,
     Stats* const parent_stats,
     shared_ptr<Logger> logger,
-    const bool coalesce_ranges,
-    StorageManager* storage_manager)
+    const bool coalesce_ranges)
     : Subarray(
           array->opened_array(),
           layout,
           parent_stats,
           logger,
-          coalesce_ranges,
-          storage_manager) {
+          coalesce_ranges) {
 }
 
 Subarray::Subarray(
@@ -127,13 +124,11 @@ Subarray::Subarray(
     const Layout layout,
     Stats* const parent_stats,
     shared_ptr<Logger> logger,
-    const bool coalesce_ranges,
-    StorageManager* storage_manager)
+    const bool coalesce_ranges)
     : stats_(
-          parent_stats ? parent_stats->create_child("Subarray") :
-          storage_manager ?
-                         storage_manager->stats()->create_child("subSubarray") :
-                         nullptr)
+          parent_stats ?
+              parent_stats->create_child("Subarray") :
+              opened_array->resources().stats().create_child("subSubarray"))
     , logger_(logger->clone("Subarray", ++logger_id_))
     , array_(opened_array)
     , layout_(layout)
@@ -141,11 +136,8 @@ Subarray::Subarray(
     , est_result_size_computed_(false)
     , relevant_fragments_(opened_array->fragment_metadata().size())
     , coalesce_ranges_(coalesce_ranges)
-    , ranges_sorted_(false) {
-  if (!parent_stats && !storage_manager) {
-    throw std::runtime_error(
-        "Subarray(): missing parent_stats requires live storage_manager!");
-  }
+    , ranges_sorted_(false)
+    , merge_overlapping_ranges_(true) {
   add_default_ranges();
 }
 
@@ -246,18 +238,11 @@ void Subarray::add_label_range(
 }
 
 void Subarray::add_label_range(
-    const std::string& label_name,
-    const void* start,
-    const void* end,
-    const void* stride) {
+    const std::string& label_name, const void* start, const void* end) {
   // Check input range data is valid data.
   if (start == nullptr || end == nullptr) {
     throw SubarrayException("[add_label_range] Invalid range");
   }
-  if (stride != nullptr) {
-    throw SubarrayException(
-        "[add_label_range] Setting range stride is currently unsupported");
-  }
   // Get dimension label range and check the label is in fact fixed-sized.
   const auto& dim_label_ref =
       array_->array_schema_latest().dimension_label(label_name);
@@ -382,8 +367,7 @@ void Subarray::set_subarray_unsafe(const void* subarray) {
   }
 }
 
-void Subarray::add_range(
-    unsigned dim_idx, const void* start, const void* end, const void* stride) {
+void Subarray::add_range(unsigned dim_idx, const void* start, const void* end) {
   if (dim_idx >= this->array_->array_schema_latest().dim_num())
     throw SubarrayException("Cannot add range; Invalid dimension index");
 
@@ -397,11 +381,6 @@ void Subarray::add_range(
     throw SubarrayException("Cannot add range; Invalid range");
   }
 
-  if (stride != nullptr) {
-    throw SubarrayException(
-        "Cannot add range; Setting range stride is currently unsupported");
-  }
-
   if (this->array_->array_schema_latest()
           .domain()
           .dimension_ptr(dim_idx)
@@ -504,13 +483,10 @@ void Subarray::add_ranges_list(
 }
 
 void Subarray::add_range_by_name(
-    const std::string& dim_name,
-    const void* start,
-    const void* end,
-    const void* stride) {
+    const std::string& dim_name, const void* start, const void* end) {
   unsigned dim_idx =
       array_->array_schema_latest().domain().get_dimension_index(dim_name);
-  add_range(dim_idx, start, end, stride);
+  add_range(dim_idx, start, end);
 }
 
 void Subarray::add_range_var(
@@ -541,17 +517,6 @@ void Subarray::add_range_var(
     throw SubarrayException("Cannot add range; Range must be variable-sized");
   }
 
-  // Get read_range_oob config setting
-  bool found = false;
-  std::string read_range_oob = config_.get("sm.read_range_oob", &found);
-  assert(found);
-
-  if (read_range_oob != "error" && read_range_oob != "warn") {
-    throw SubarrayException(
-        "Invalid value " + read_range_oob +
-        " for sm.read_range_obb. Acceptable values are 'error' or 'warn'.");
-  }
-
   // Add range
   this->add_range(
       dim_idx, Range(start, start_size, end, end_size), err_on_range_oob_);
@@ -585,8 +550,7 @@ void Subarray::get_label_range(
     const std::string& label_name,
     uint64_t range_idx,
     const void** start,
-    const void** end,
-    const void** stride) const {
+    const void** end) const {
   auto dim_idx = array_->array_schema_latest()
                      .dimension_label(label_name)
                      .dimension_index();
@@ -599,7 +563,6 @@ void Subarray::get_label_range(
   const auto& range = label_range_subset_[dim_idx].value().ranges_[range_idx];
   *start = range.start_fixed();
   *end = range.end_fixed();
-  *stride = nullptr;
 }
 
 void Subarray::get_label_range_num(
@@ -659,8 +622,7 @@ void Subarray::get_range_var(
 
   const void* range_start;
   const void* range_end;
-  const void* stride;
-  get_range(dim_idx, range_idx, &range_start, &range_end, &stride);
+  get_range(dim_idx, range_idx, &range_start, &range_end);
 
   std::memcpy(start, range_start, start_size);
   std::memcpy(end, range_end, end_size);
@@ -673,25 +635,14 @@ void Subarray::get_range_num_from_name(
   get_range_num(dim_idx, range_num);
 }
 
-void Subarray::get_range(
-    unsigned dim_idx,
-    uint64_t range_idx,
-    const void** start,
-    const void** end,
-    const void** stride) const {
-  *stride = nullptr;
-  this->get_range(dim_idx, range_idx, start, end);
-}
-
 void Subarray::get_range_from_name(
     const std::string& dim_name,
     uint64_t range_idx,
     const void** start,
-    const void** end,
-    const void** stride) const {
+    const void** end) const {
   unsigned dim_idx =
       array_->array_schema_latest().domain().get_dimension_index(dim_name);
-  get_range(dim_idx, range_idx, start, end, stride);
+  get_range(dim_idx, range_idx, start, end);
 }
 
 void Subarray::get_range_var_size_from_name(
@@ -854,17 +805,52 @@ template <class T>
 Subarray Subarray::crop_to_tile(const T* tile_coords, Layout layout) const {
   // TBD: is it ok that Subarray log id will increase as if it's a new subarray?
   Subarray ret(array_, layout, stats_->parent(), logger_, false);
-  crop_to_tile_impl(tile_coords, ret);
+  crop_to_tile_impl<T, Subarray>(tile_coords, ret);
 
   return ret;
 }
 
 template <class T>
 void Subarray::crop_to_tile(
-    Subarray* ret, const T* tile_coords, Layout layout) const {
-  new (ret) Subarray(array_, layout, stats_->parent(), logger_, false);
-  Subarray(array_, layout, stats_->parent(), logger_, false);
-  crop_to_tile_impl(tile_coords, *ret);
+    DenseTileSubarray<T>& ret, const T* tile_coords) const {
+  crop_to_tile_impl<T, DenseTileSubarray<T>>(tile_coords, ret);
+}
+
+template <class T>
+uint64_t Subarray::tile_cell_num(const T* tile_coords) const {
+  uint64_t ret = 1;
+  T new_range[2];
+  bool overlaps;
+
+  // Get tile subarray based on the input coordinates
+  const auto& array_schema = array_->array_schema_latest();
+  std::vector<T> tile_subarray(2 * dim_num());
+  array_schema.domain().get_tile_subarray(tile_coords, &tile_subarray[0]);
+
+  // Compute cell num per dims.
+  for (unsigned d = 0; d < dim_num(); ++d) {
+    uint64_t cell_num_for_dim = 0;
+    for (size_t r = 0; r < range_subset_[d].num_ranges(); ++r) {
+      const auto& range = range_subset_[d][r];
+      rectangle::overlap(
+          (const T*)range.data(),
+          &tile_subarray[2 * d],
+          1,
+          new_range,
+          &overlaps);
+
+      if (overlaps) {
+        // Here we need to do +1 because both the start and end are included in
+        // the number of cells.
+        cell_num_for_dim +=
+            static_cast<uint64_t>(new_range[1] - new_range[0] + 1);
+      }
+    }
+
+    ret *= cell_num_for_dim;
+  }
+
+  return ret;
 }
 
 uint32_t Subarray::dim_num() const {
@@ -1058,7 +1044,9 @@ void Subarray::set_layout(Layout layout) {
 }
 
 void Subarray::set_config(const QueryType query_type, const Config& config) {
-  config_.inherit(config);
+  merge_overlapping_ranges_ = config.get<bool>(
+      "sm.merge_overlapping_ranges_experimental", Config::MustFindMarker());
+
   if (query_type == QueryType::READ) {
     bool found = false;
     std::string read_range_oob_str = config.get("sm.read_range_oob", &found);
@@ -1071,10 +1059,6 @@ void Subarray::set_config(const QueryType query_type, const Config& config) {
   }
 };
 
-const Config* Subarray::config() const {
-  return &config_;
-}
-
 void Subarray::set_coalesce_ranges(bool coalesce_ranges) {
   if (count_set_ranges()) {
     throw SubarrayException(
@@ -1104,39 +1088,45 @@ Layout Subarray::layout() const {
 }
 
 FieldDataSize Subarray::get_est_result_size(
-    const char* name, const Config* config, ThreadPool* compute_tp) {
-  // Check attribute/dimension name
-  if (name == nullptr) {
-    throw SubarrayException(
-        "Cannot get estimated result size; field name cannot be null");
+    std::string_view field_name, const Config* config, ThreadPool* compute_tp) {
+  /*
+   * This check throws a logic error because we expect the field name to have
+   * already been validated in the C API.
+   */
+  if (field_name.data() == nullptr) {
+    throw std::logic_error(
+        "Cannot get estimated result size; field name is null");
   }
 
   // Check if name is attribute or dimension
   const auto& array_schema = array_->array_schema_latest();
-  const bool is_dim = array_schema.is_dim(name);
-  const bool is_attr = array_schema.is_attr(name);
+  const bool is_dim = array_schema.is_dim(field_name.data());
+  const bool is_attr = array_schema.is_attr(field_name.data());
 
   // Check if attribute/dimension exists
-  if (!ArraySchema::is_special_attribute(name) && !is_dim && !is_attr) {
+  if (!ArraySchema::is_special_attribute(field_name.data()) && !is_dim &&
+      !is_attr) {
     throw SubarrayException(
         std::string("Cannot get estimated result size; ") +
-        "there is no field named '" + name + "'");
+        "there is no field named '" + std::string(field_name) + "'");
   }
 
   bool is_variable_sized{
-      name != constants::coords && array_schema.var_size(name)};
-  bool is_nullable{array_schema.is_nullable(name)};
+      field_name != constants::coords &&
+      array_schema.var_size(field_name.data())};
+  bool is_nullable{array_schema.is_nullable(field_name.data())};
 
   // Compute tile overlap for each fragment
   compute_est_result_size(config, compute_tp);
 
   FieldDataSize r{
-      static_cast<size_t>(std::ceil(est_result_size_[name].size_fixed_)),
-      is_variable_sized ?
-          static_cast<size_t>(std::ceil(est_result_size_[name].size_var_)) :
-          0,
-      is_nullable ? static_cast<size_t>(
-                        std::ceil(est_result_size_[name].size_validity_)) :
+      static_cast<size_t>(
+          std::ceil(est_result_size_[field_name.data()].size_fixed_)),
+      is_variable_sized ? static_cast<size_t>(std::ceil(
+                              est_result_size_[field_name.data()].size_var_)) :
+                          0,
+      is_nullable ? static_cast<size_t>(std::ceil(
+                        est_result_size_[field_name.data()].size_validity_)) :
                     0};
   /*
    * Special fix-ups may be necessary if data is empty or very short.
@@ -1154,7 +1144,8 @@ FieldDataSize Subarray::get_est_result_size(
         r.fixed_ = off_cell_size;
       }
       // Ensure that there's space for at least one data value.
-      const auto val_cell_size = datatype_size(array_schema.type(name));
+      const auto val_cell_size =
+          datatype_size(array_schema.type(field_name.data()));
       if (r.variable_ < val_cell_size) {
         r.variable_ = val_cell_size;
       }
@@ -1170,7 +1161,7 @@ FieldDataSize Subarray::get_est_result_size(
      * If the fixed data is not empty, ensure it is large enough to contain at
      * least one cell.
      */
-    const auto cell_size = array_schema.cell_size(name);
+    const auto cell_size = array_schema.cell_size(field_name.data());
     if (0 < r.fixed_ && r.fixed_ < cell_size) {
       r.fixed_ = cell_size;
       if (is_nullable) {
@@ -1595,7 +1586,8 @@ void Subarray::compute_relevant_fragment_est_result_sizes(
                   tile_size / cell_size * constants::cell_validity_size;
           } else {
             tile_size -= constants::cell_var_offset_size;
-            auto tile_var_size = meta->tile_var_size(names[i], ft.second);
+            auto tile_var_size =
+                meta->loaded_metadata()->tile_var_size(names[i], ft.second);
             mem_vec[i].size_fixed_ += tile_size;
             mem_vec[i].size_var_ += tile_var_size;
             if (nullable[i])
@@ -1641,9 +1633,6 @@ void Subarray::sort_and_merge_ranges(ThreadPool* const compute_tp) {
   if (ranges_sorted_)
     return;
 
-  auto merge = config_.get<bool>(
-      "sm.merge_overlapping_ranges_experimental", Config::MustFindMarker());
-
   // Sort and conditionally merge ranges
   auto timer = stats_->start_timer("sort_and_merge_ranges");
   throw_if_not_ok(parallel_for(
@@ -1651,7 +1640,8 @@ void Subarray::sort_and_merge_ranges(ThreadPool* const compute_tp) {
       0,
       array_->array_schema_latest().dim_num(),
       [&](uint64_t dim_idx) {
-        range_subset_[dim_idx].sort_and_merge_ranges(compute_tp, merge);
+        range_subset_[dim_idx].sort_and_merge_ranges(
+            compute_tp, merge_overlapping_ranges_);
         return Status::Ok();
       }));
   ranges_sorted_ = true;
@@ -1919,11 +1909,12 @@ void Subarray::compute_relevant_fragment_est_result_sizes(
           } else {
             tile_size -= constants::cell_var_offset_size;
             (*result_sizes)[n].size_fixed_ += tile_size;
-            auto tile_var_size = meta->tile_var_size(name[n], tid);
+            auto tile_var_size =
+                meta->loaded_metadata()->tile_var_size(name[n], tid);
             (*result_sizes)[n].size_var_ += tile_var_size;
             if (nullable[n])
               (*result_sizes)[n].size_validity_ +=
-                  tile_var_size / attr_datatype_size *
+                  tile_size / constants::cell_var_offset_size *
                   constants::cell_validity_size;
           }
         }
@@ -1959,11 +1950,12 @@ void Subarray::compute_relevant_fragment_est_result_sizes(
         } else {
           tile_size -= constants::cell_var_offset_size;
           (*result_sizes)[n].size_fixed_ += tile_size * ratio;
-          auto tile_var_size = meta->tile_var_size(name[n], tid);
+          auto tile_var_size =
+              meta->loaded_metadata()->tile_var_size(name[n], tid);
           (*result_sizes)[n].size_var_ += tile_var_size * ratio;
           if (nullable[n])
             (*result_sizes)[n].size_validity_ +=
-                (tile_var_size / attr_datatype_size *
+                (tile_size / constants::cell_var_offset_size *
                  constants::cell_validity_size) *
                 ratio;
         }
@@ -2288,7 +2280,7 @@ void Subarray::precompute_all_ranges_tile_overlap(
                 const auto r_end =
                     std::min((t + 1) * ranges_per_thread - 1, range_num - 1);
                 for (uint64_t r = r_start; r <= r_end; ++r) {
-                  meta[f]->compute_tile_bitmap(
+                  meta[f]->loaded_metadata()->compute_tile_bitmap(
                       range_subset_[d][r], d, &tile_bitmaps[d]);
                 }
                 return Status::Ok();
@@ -2349,6 +2341,7 @@ Subarray Subarray::clone() const {
   clone.est_result_size_ = est_result_size_;
   clone.max_mem_size_ = max_mem_size_;
   clone.original_range_idx_ = original_range_idx_;
+  clone.merge_overlapping_ranges_ = merge_overlapping_ranges_;
 
   return clone;
 }
@@ -2459,6 +2452,7 @@ void Subarray::swap(Subarray& subarray) {
   std::swap(est_result_size_, subarray.est_result_size_);
   std::swap(max_mem_size_, subarray.max_mem_size_);
   std::swap(original_range_idx_, subarray.original_range_idx_);
+  std::swap(merge_overlapping_ranges_, subarray.merge_overlapping_ranges_);
 }
 
 void Subarray::get_expanded_coordinates(
@@ -2542,7 +2536,8 @@ void Subarray::load_relevant_fragment_rtrees(ThreadPool* compute_tp) const {
 
   auto status =
       parallel_for(compute_tp, 0, relevant_fragments_.size(), [&](uint64_t f) {
-        meta[relevant_fragments_[f]]->load_rtree(*encryption_key);
+        meta[relevant_fragments_[f]]->loaded_metadata()->load_rtree(
+            *encryption_key);
         return Status::Ok();
       });
   throw_if_not_ok(status);
@@ -2560,48 +2555,42 @@ void Subarray::compute_relevant_fragment_tile_overlap(
 
   const auto& meta = array_->fragment_metadata();
 
-  auto status =
-      parallel_for(compute_tp, 0, relevant_fragments_.size(), [&](uint64_t i) {
-        const auto f = relevant_fragments_[i];
-        const auto dense = meta[f]->dense();
-        compute_relevant_fragment_tile_overlap(
-            meta[f], f, dense, compute_tp, tile_overlap, fn_ctx);
+  const auto num_threads = compute_tp->concurrency_level();
+  const auto ranges_per_thread =
+      (uint64_t)std::ceil((double)fn_ctx->range_len_ / num_threads);
+
+  auto status = parallel_for_2d(
+      compute_tp,
+      0,
+      relevant_fragments_.size(),
+      0,
+      num_threads,
+      [&](uint64_t i, uint64_t t) {
+        const auto frag_idx = relevant_fragments_[i];
+        const auto dense = meta[frag_idx]->dense();
+
+        const auto r_start =
+            fn_ctx->range_idx_offset_ + (t * ranges_per_thread);
+        const auto r_end =
+            fn_ctx->range_idx_offset_ +
+            std::min((t + 1) * ranges_per_thread - 1, fn_ctx->range_len_ - 1);
+        for (uint64_t r = r_start; r <= r_end; ++r) {
+          if (dense) {  // Dense fragment
+            *tile_overlap->at(frag_idx, r) = compute_tile_overlap(
+                r + tile_overlap->range_idx_start(), frag_idx);
+          } else {  // Sparse fragment
+            const auto& range =
+                this->ndrange(r + tile_overlap->range_idx_start());
+            meta[frag_idx]->loaded_metadata()->get_tile_overlap(
+                range, is_default_, tile_overlap->at(frag_idx, r));
+          }
+        }
+
         return Status::Ok();
       });
   throw_if_not_ok(status);
 }
 
-void Subarray::compute_relevant_fragment_tile_overlap(
-    shared_ptr<FragmentMetadata> meta,
-    unsigned frag_idx,
-    bool dense,
-    ThreadPool* compute_tp,
-    SubarrayTileOverlap* tile_overlap,
-    ComputeRelevantTileOverlapCtx* fn_ctx) {
-  const auto num_threads = compute_tp->concurrency_level();
-  const auto range_num = fn_ctx->range_len_;
-
-  const auto ranges_per_thread =
-      (uint64_t)std::ceil((double)range_num / num_threads);
-  const auto status = parallel_for(compute_tp, 0, num_threads, [&](uint64_t t) {
-    const auto r_start = fn_ctx->range_idx_offset_ + (t * ranges_per_thread);
-    const auto r_end = fn_ctx->range_idx_offset_ +
-                       std::min((t + 1) * ranges_per_thread - 1, range_num - 1);
-    for (uint64_t r = r_start; r <= r_end; ++r) {
-      if (dense) {  // Dense fragment
-        *tile_overlap->at(frag_idx, r) =
-            compute_tile_overlap(r + tile_overlap->range_idx_start(), frag_idx);
-      } else {  // Sparse fragment
-        const auto& range = this->ndrange(r + tile_overlap->range_idx_start());
-        meta->get_tile_overlap(
-            range, is_default_, tile_overlap->at(frag_idx, r));
-      }
-    }
-    return Status::Ok();
-  });
-  throw_if_not_ok(status);
-}
-
 void Subarray::load_relevant_fragment_tile_var_sizes(
     const std::vector<std::string>& names, ThreadPool* compute_tp) const {
   const auto& array_schema = array_->array_schema_latest();
@@ -2632,7 +2621,8 @@ void Subarray::load_relevant_fragment_tile_var_sizes(
           if (!schema->is_field(var_name)) {
             return Status::Ok();
           }
-          meta[f]->load_tile_var_sizes(*encryption_key, var_name);
+          meta[f]->loaded_metadata()->load_tile_var_sizes(
+              *encryption_key, var_name);
           return Status::Ok();
         });
     throw_if_not_ok(status);
@@ -2668,8 +2658,8 @@ bool Subarray::non_overlapping_ranges_for_dim(const uint64_t dim_idx) {
   return true;
 }
 
-template <class T>
-void Subarray::crop_to_tile_impl(const T* tile_coords, Subarray& ret) const {
+template <class T, class SubarrayT>
+void Subarray::crop_to_tile_impl(const T* tile_coords, SubarrayT& ret) const {
   T new_range[2];
   bool overlaps;
 
@@ -2684,7 +2674,7 @@ void Subarray::crop_to_tile_impl(const T* tile_coords, Subarray& ret) const {
     uint64_t i = 0;
     for (size_t r = 0; r < range_subset_[d].num_ranges(); ++r) {
       const auto& range = range_subset_[d][r];
-      utils::geometry::overlap(
+      rectangle::overlap(
           (const T*)range.data(),
           &tile_subarray[2 * d],
           1,
@@ -2693,9 +2683,9 @@ void Subarray::crop_to_tile_impl(const T* tile_coords, Subarray& ret) const {
 
       if (overlaps) {
         ret.add_range_unsafe(d, Range(new_range, r_size));
-        ret.original_range_idx_.resize(dim_num());
-        ret.original_range_idx_[d].resize(i + 1);
-        ret.original_range_idx_[d][i++] = r;
+        ret.original_range_idx_unsafe().resize(dim_num());
+        ret.original_range_idx_unsafe()[d].resize(i + 1);
+        ret.original_range_idx_unsafe()[d][i++] = r;
       }
     }
   }
@@ -2766,25 +2756,38 @@ template Subarray Subarray::crop_to_tile<double>(
     const double* tile_coords, Layout layout) const;
 
 template void Subarray::crop_to_tile<int8_t>(
-    Subarray* ret, const int8_t* tile_coords, Layout layout) const;
+    DenseTileSubarray<int8_t>& ret, const int8_t* tile_coords) const;
 template void Subarray::crop_to_tile<uint8_t>(
-    Subarray* ret, const uint8_t* tile_coords, Layout layout) const;
+    DenseTileSubarray<uint8_t>& ret, const uint8_t* tile_coords) const;
 template void Subarray::crop_to_tile<int16_t>(
-    Subarray* ret, const int16_t* tile_coords, Layout layout) const;
+    DenseTileSubarray<int16_t>& ret, const int16_t* tile_coords) const;
 template void Subarray::crop_to_tile<uint16_t>(
-    Subarray* ret, const uint16_t* tile_coords, Layout layout) const;
+    DenseTileSubarray<uint16_t>& ret, const uint16_t* tile_coords) const;
 template void Subarray::crop_to_tile<int32_t>(
-    Subarray* ret, const int32_t* tile_coords, Layout layout) const;
+    DenseTileSubarray<int32_t>& ret, const int32_t* tile_coords) const;
 template void Subarray::crop_to_tile<uint32_t>(
-    Subarray* ret, const uint32_t* tile_coords, Layout layout) const;
+    DenseTileSubarray<uint32_t>& ret, const uint32_t* tile_coords) const;
 template void Subarray::crop_to_tile<int64_t>(
-    Subarray* ret, const int64_t* tile_coords, Layout layout) const;
+    DenseTileSubarray<int64_t>& ret, const int64_t* tile_coords) const;
 template void Subarray::crop_to_tile<uint64_t>(
-    Subarray* ret, const uint64_t* tile_coords, Layout layout) const;
-template void Subarray::crop_to_tile<float>(
-    Subarray* ret, const float* tile_coords, Layout layout) const;
-template void Subarray::crop_to_tile<double>(
-    Subarray* ret, const double* tile_coords, Layout layout) const;
+    DenseTileSubarray<uint64_t>& ret, const uint64_t* tile_coords) const;
+
+template uint64_t Subarray::tile_cell_num<int8_t>(
+    const int8_t* tile_coords) const;
+template uint64_t Subarray::tile_cell_num<uint8_t>(
+    const uint8_t* tile_coords) const;
+template uint64_t Subarray::tile_cell_num<int16_t>(
+    const int16_t* tile_coords) const;
+template uint64_t Subarray::tile_cell_num<uint16_t>(
+    const uint16_t* tile_coords) const;
+template uint64_t Subarray::tile_cell_num<int32_t>(
+    const int32_t* tile_coords) const;
+template uint64_t Subarray::tile_cell_num<uint32_t>(
+    const uint32_t* tile_coords) const;
+template uint64_t Subarray::tile_cell_num<int64_t>(
+    const int64_t* tile_coords) const;
+template uint64_t Subarray::tile_cell_num<uint64_t>(
+    const uint64_t* tile_coords) const;
 
 /* ********************************* */
 /*         LABEL RANGE SUBSET        */
