@@ -5,7 +5,7 @@
  *
  * The MIT License
  *
- * @copyright Copyright (c) 2017-2023 TileDB, Inc.
+ * @copyright Copyright (c) 2017-2024 TileDB, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -36,14 +36,14 @@
 #include <atomic>
 
 #include "tiledb/common/common.h"
-#include "tiledb/common/thread_pool.h"
+#include "tiledb/common/pmr.h"
+#include "tiledb/common/thread_pool/thread_pool.h"
 #include "tiledb/sm/buffer/buffer.h"
 #include "tiledb/sm/config/config.h"
 #include "tiledb/sm/enums/datatype.h"
 #include "tiledb/sm/misc/tile_overlap.h"
 #include "tiledb/sm/misc/types.h"
 #include "tiledb/sm/stats/stats.h"
-#include "tiledb/sm/storage_manager/storage_manager_declaration.h"
 #include "tiledb/sm/subarray/range_subset.h"
 #include "tiledb/sm/subarray/relevant_fragments.h"
 #include "tiledb/sm/subarray/subarray_tile_overlap.h"
@@ -93,6 +93,114 @@ class FragmentMetadata;
 enum class Layout : uint8_t;
 enum class QueryType : uint8_t;
 
+template <class T>
+class DenseTileSubarray {
+ public:
+  /**
+   * Stores information about a range along a single dimension. The
+   * whole range resides in a single tile.
+   */
+  struct DenseTileRange {
+    /** The start of the range in global coordinates. */
+    T start_;
+    /** The end of the range in global coordinates. */
+    T end_;
+
+    /** Constructor. */
+    DenseTileRange(T start, T end)
+        : start_(start)
+        , end_(end) {
+    }
+
+    /** Equality operator. */
+    bool operator==(const DenseTileRange& r) const {
+      return (r.start_ == start_ && r.end_ == end_);
+    }
+  };
+
+  /**
+   * The type of the allocator. Required to make the type allocator-aware.
+   *
+   * uint8_t was arbitrarily chosen and does not matter; allocators can convert
+   * between any types.
+   */
+  using allocator_type = tdb::pmr::polymorphic_allocator<uint8_t>;
+
+  /* ********************************* */
+  /*     CONSTRUCTORS & DESTRUCTORS    */
+  /* ********************************* */
+
+  DenseTileSubarray() = delete;
+
+  DenseTileSubarray(unsigned dim_num, const allocator_type& alloc = {})
+      : original_range_idx_(alloc)
+      , ranges_(dim_num, alloc) {
+  }
+
+  /** Default copy constructor. */
+  DenseTileSubarray(const DenseTileSubarray&) = default;
+  /** Default move constructor. */
+  DenseTileSubarray(DenseTileSubarray&&) = default;
+
+  /** Allocator-aware copy constructor. */
+  DenseTileSubarray(const DenseTileSubarray& other, const allocator_type& alloc)
+      : original_range_idx_(other.original_range_idx_, alloc)
+      , ranges_(other.ranges_, alloc) {
+  }
+
+  /** Allocator-aware move constructor. */
+  DenseTileSubarray(DenseTileSubarray&& other, const allocator_type& alloc)
+      : original_range_idx_(std::move(other.original_range_idx_), alloc)
+      , ranges_(std::move(other.ranges_), alloc) {
+  }
+
+  /** Default copy-assign operator. */
+  DenseTileSubarray& operator=(const DenseTileSubarray&) = default;
+  /** Default move-assign operator. */
+  DenseTileSubarray& operator=(DenseTileSubarray&&) = default;
+
+  /* ********************************* */
+  /*                 API               */
+  /* ********************************* */
+
+  /** Returns the orignal range indexes. */
+  inline const tdb::pmr::vector<tdb::pmr::vector<uint64_t>>&
+  original_range_idx() const {
+    return original_range_idx_;
+  }
+
+  /** Returns the ranges. */
+  inline const tdb::pmr::vector<tdb::pmr::vector<DenseTileRange>>& ranges()
+      const {
+    return ranges_;
+  }
+
+  /** Returns the orignal range indexes to be modified. */
+  inline tdb::pmr::vector<tdb::pmr::vector<uint64_t>>&
+  original_range_idx_unsafe() {
+    return original_range_idx_;
+  }
+
+  /**
+   * Adds a range along the dimension with the given index, without
+   * performing any error checks.
+   */
+  void add_range_unsafe(uint32_t dim_idx, const Range& range) {
+    ranges_[dim_idx].emplace_back(range.start_as<T>(), range.end_as<T>());
+  }
+
+ private:
+  /* ********************************* */
+  /*         PRIVATE ATTRIBUTES        */
+  /* ********************************* */
+
+  /** Stores a vector of 1D ranges per dimension. */
+  tdb::pmr::vector<tdb::pmr::vector<uint64_t>> original_range_idx_;
+
+  /** Stores the ranges per dimension. */
+  tdb::pmr::vector<tdb::pmr::vector<DenseTileRange>> ranges_;
+};
+
 /**
  * Interface to implement for a class that can store tile ranges computed by
  * this class.
@@ -294,8 +402,7 @@ class Subarray {
       const Array* array,
       stats::Stats* parent_stats,
       shared_ptr<Logger> logger,
-      bool coalesce_ranges = true,
-      StorageManager* storage_manager = nullptr);
+      bool coalesce_ranges = true);
 
   /**
    * Constructor.
@@ -314,8 +421,7 @@ class Subarray {
       Layout layout,
       stats::Stats* parent_stats,
       shared_ptr<Logger> logger,
-      bool coalesce_ranges = true,
-      StorageManager* storage_manager = nullptr);
+      bool coalesce_ranges = true);
 
   /**
    * Constructor.
@@ -334,8 +440,7 @@ class Subarray {
       Layout layout,
       stats::Stats* parent_stats,
       shared_ptr<Logger> logger,
-      bool coalesce_ranges = true,
-      StorageManager* storage_manager = nullptr);
+      bool coalesce_ranges = true);
 
   /**
    * Copy constructor. This performs a deep copy (including memcpy of
@@ -365,12 +470,6 @@ class Subarray {
   /** Sets config for query-level parameters only. */
   void set_config(const QueryType query_type, const Config& config);
 
-  /**
-   * Get the config of the writer
-   * @return Config
-   */
-  const Config* config() const;
-
   /**
    * Sets the subarray using a pointer to raw range data that stores one range
    * per dimension.
@@ -429,13 +528,9 @@ class Subarray {
    * @param label_name The name of the dimension label to add the range to.
    * @param start The range start.
    * @param end The range end.
-   * @param stride The range stride.
    */
   void add_label_range(
-      const std::string& label_name,
-      const void* start,
-      const void* end,
-      const void* stride);
+      const std::string& label_name, const void* start, const void* end);
 
   /**
    * Adds a variable-sized range along the dimension with the given index for
@@ -459,13 +554,11 @@ class Subarray {
       uint32_t dim_idx, Range&& range, const bool read_range_oob_error = true);
 
   /**
-   * Adds a range to the subarray on the input dimension by index,
-   * in the form of (start, end, stride).
+   * Adds a range to the subarray on the input dimension by index.
    * The range components must be of the same type as the domain type of the
    * underlying array.
    */
-  void add_range(
-      unsigned dim_idx, const void* start, const void* end, const void* stride);
+  void add_range(unsigned dim_idx, const void* start, const void* end);
 
   /**
    * @brief Set point ranges from an array
@@ -514,6 +607,11 @@ class Subarray {
       const void* end,
       uint64_t end_size);
 
+  /** Returns the orignal range indexes to be modified. */
+  inline std::vector<std::vector<uint64_t>>& original_range_idx_unsafe() {
+    return original_range_idx_;
+  }
+
   /**
    * Adds a range along the dimension with the given index, without
    * performing any error checks.
@@ -521,16 +619,12 @@ class Subarray {
   void add_range_unsafe(uint32_t dim_idx, const Range& range);
 
   /**
-   * Adds a range to the (read/write) query on the input dimension by name,
-   * in the form of (start, end, stride).
+   * Adds a range to the (read/write) query on the input dimension by name.
    * The range components must be of the same type as the domain type of the
    * underlying array.
    */
   void add_range_by_name(
-      const std::string& dim_name,
-      const void* start,
-      const void* end,
-      const void* stride);
+      const std::string& dim_name, const void* start, const void* end);
 
   /**
    * Adds a variable-sized range to the (read/write) query on the input
@@ -567,22 +661,19 @@ class Subarray {
   const std::string& get_label_name(const uint32_t dim_index) const;
 
   /**
-   * Retrieves a range from a dimension label name in the form (start, end,
-   * stride).
+   * Retrieves a range from a dimension label name.
    *
    * @param label_name The name of the dimension label to retrieve the range
    *     from.
    * @param range_idx The id of the range to retrieve.
    * @param start The range start to retrieve.
    * @param end The range end to retrieve.
-   * @param stride The range stride to retrieve.
    */
   void get_label_range(
       const std::string& label_name,
       uint64_t range_idx,
       const void** start,
-      const void** end,
-      const void** stride) const;
+      const void** end) const;
 
   /**
    * Retrieves the number of ranges of the subarray for the given dimension
@@ -633,20 +724,18 @@ class Subarray {
       const std::string& dim_name, uint64_t* range_num) const;
 
   /**
-   * Retrieves a range from a dimension name in the form (start, end, stride).
+   * Retrieves a range from a dimension name.
    *
    * @param dim_name The dimension to retrieve the range from.
    * @param range_idx The id of the range to retrieve.
    * @param start The range start to retrieve.
    * @param end The range end to retrieve.
-   * @param stride The range stride to retrieve.
    */
   void get_range_from_name(
       const std::string& dim_name,
       uint64_t range_idx,
       const void** start,
-      const void** end,
-      const void** stride) const;
+      const void** end) const;
 
   /**
    * Retrieves a range's sizes for a variable-length dimension name
@@ -822,7 +911,13 @@ class Subarray {
    * the input `layout`.
    */
   template <class T>
-  void crop_to_tile(Subarray* ret, const T* tile_coords, Layout layout) const;
+  void crop_to_tile(DenseTileSubarray<T>& ret, const T* tile_coords) const;
+
+  /**
+   * Returns the number of cells in a specific tile for this subarray.
+   */
+  template <class T>
+  uint64_t tile_cell_num(const T* tile_coords) const;
 
   /** Returns the number of dimensions of the subarray. */
   uint32_t dim_num() const;
@@ -895,22 +990,6 @@ class Subarray {
   /** Retrieves the number of ranges on the given dimension index. */
   void get_range_num(uint32_t dim_idx, uint64_t* range_num) const;
 
-  /**
-   * Retrieves a range from a dimension index in the form (start, end, stride).
-   *
-   * @param dim_idx The dimension to retrieve the range from.
-   * @param range_idx The id of the range to retrieve.
-   * @param start The range start to retrieve.
-   * @param end The range end to retrieve.
-   * @param stride The range stride to retrieve.
-   */
-  void get_range(
-      unsigned dim_idx,
-      uint64_t range_idx,
-      const void** start,
-      const void** end,
-      const void** stride) const;
-
   /**
    * ``True`` if the specified dimension is set by default.
    *
@@ -946,7 +1025,9 @@ class Subarray {
    * size is not relevant to the field type, then it's returned as zero.
    */
   FieldDataSize get_est_result_size(
-      const char* name, const Config* config, ThreadPool* compute_tp);
+      std::string_view field_name,
+      const Config* config,
+      ThreadPool* compute_tp);
 
   /**
    * The maximum memory in bytes required for a field.
@@ -1457,9 +1538,6 @@ class Subarray {
   std::unordered_map<std::vector<uint8_t>, size_t, CoordsHasher>
       tile_coords_map_;
 
-  /** The config for query-level parameters only. */
-  Config config_;
-
   /** State of specific Config item needed from multiple locations. */
   bool err_on_range_oob_ = true;
 
@@ -1469,6 +1547,9 @@ class Subarray {
   /** Mutext to protect sorting ranges. */
   std::mutex ranges_sort_mtx_;
 
+  /** Merge overlapping ranges setting. */
+  bool merge_overlapping_ranges_;
+
   /* ********************************* */
   /*           PRIVATE METHODS         */
   /* ********************************* */
@@ -1546,25 +1627,6 @@ class Subarray {
       SubarrayTileOverlap* tile_overlap,
       ComputeRelevantTileOverlapCtx* fn_ctx);
 
-  /**
-   * Computes the tile overlap for all ranges on the given relevant fragment.
-   *
-   * @param meta The fragment metadat to focus on.
-   * @param frag_idx The fragment id.
-   * @param dense Whether the fragment is dense or sparse.
-   * @param compute_tp The thread pool for compute-bound tasks.
-   * @param tile_overlap Mutated to store the computed tile overlap.
-   * @param fn_ctx An opaque context object to be used between successive
-   * invocations.
-   */
-  void compute_relevant_fragment_tile_overlap(
-      shared_ptr<FragmentMetadata> meta,
-      unsigned frag_idx,
-      bool dense,
-      ThreadPool* compute_tp,
-      SubarrayTileOverlap* tile_overlap,
-      ComputeRelevantTileOverlapCtx* fn_ctx);
-
   /**
    * Load the var-sized tile sizes for the input names and from the
    * relevant fragments.
@@ -1585,8 +1647,8 @@ class Subarray {
    * tile with the input coordinates. The new subarray will have
    * the input `layout`.
    */
-  template <class T>
-  void crop_to_tile_impl(const T* tile_coords, Subarray& ret) const;
+  template <class T, class SubarrayT>
+  void crop_to_tile_impl(const T* tile_coords, SubarrayT& ret) const;
 };
 
 }  // namespace tiledb::sm
